@@ -1,6 +1,7 @@
 /* address.c
+ * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +18,8 @@
  */
 #include "config.h"
 #include "i18n.h"
+#include <stdlib.h>
+#include <string.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <pi-source.h>
@@ -30,10 +33,6 @@
 #include "libplugin.h"
 #include "password.h"
 
-#include "japanese.h"
-#include "cp1250.h"
-#include "russian.h"
-
 #define ADDRESS_EOF 7
 
 static int glob_sort_by_company;
@@ -61,7 +60,7 @@ int address_compare(const void *v1, const void *v2)
 
    al1=(AddressList **)v1;
    al2=(AddressList **)v2;
-   
+
    a1=&((*al1)->ma.a);
    a2=&((*al2)->ma.a);
 
@@ -77,9 +76,9 @@ int address_compare(const void *v1, const void *v2)
    /*sort_by_company: */
    /*0 last, first or */
    /*1 company, last */
-   
+
    str1=str2=NULL;
-   
+
    if (a1->entry[sort1] || a1->entry[sort2]) {
       if (a1->entry[sort1] && a1->entry[sort2]) {
 	 if ((str1 = (char *)malloc(strlen(a1->entry[sort1])+strlen(a1->entry[sort2])+1)) == NULL) {
@@ -189,7 +188,7 @@ int address_sort(AddressList **al, int sort_order)
       jpilot_logf(LOG_WARN, "address_sort(): Out of Memory\n");
       return 0;
    }
-   
+
    /* Set our array to be a list of pointers to the nodes in the linked list */
    for (i=0, temp_al=*al; temp_al; temp_al=temp_al->next, i++) {
       sort_al[i] = temp_al;
@@ -223,8 +222,16 @@ int pc_address_write(struct Address *a, PCRecType rt, unsigned char attrib,
 		     unsigned int *unique_id)
 {
    char record[65536];
-   int rec_len;
+   int rec_len,i;
    buf_rec br;
+   long char_set;
+
+   get_pref(PREF_CHAR_SET, &char_set, NULL);
+   if (char_set != CHAR_SET_ENGLISH) {
+      for (i = 0; i < 19; i++) {
+	 if (a->entry[i]) charset_j2p(a->entry[i], strlen(a->entry[i])+1, char_set);
+      }
+   }
 
    rec_len = pack_Address(a, record, 65535);
    if (!rec_len) {
@@ -236,10 +243,10 @@ int pc_address_write(struct Address *a, PCRecType rt, unsigned char attrib,
    br.attrib = attrib;
    br.buf = record;
    br.size = rec_len;
-   
+
    jp_pc_write("AddressDB", &br);
    *unique_id = br.unique_id;
-   
+
    return 0;
 }
 
@@ -273,36 +280,22 @@ int get_address_app_info(struct AddressAppInfo *ai)
       jpilot_logf(LOG_WARN, _("Error reading"), "AddressDB.pdb");
       return -1;
    }
-	 
-   get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set==CHAR_SET_JAPANESE || 
-       char_set==CHAR_SET_1250 ||
-       char_set==CHAR_SET_1251 ||
-       char_set==CHAR_SET_1251_B
-       ) {
+
+   get_pref(PREF_CHAR_SET, &char_set, NULL); 
+   if (char_set != CHAR_SET_ENGLISH) {
       /* Convert to character set */
-	{
-	   int i;
-	   for (i = 0; i < 16; i++)
-	     if (ai->category.name[i][0] != '\0') {
-   		if (char_set==CHAR_SET_JAPANESE) Sjis2Euc(ai->category.name[i], 16);
-   		if (char_set==CHAR_SET_1250) Win2Lat(ai->category.name[i], 16);
-   		if (char_set==CHAR_SET_1251) win1251_to_koi8(ai->category.name[i], 16);
-	     }
-	   for (i = 0; i < 19 + 3; i++)
-	     if (ai->labels[i][0] != '\0') {
-   		if (char_set==CHAR_SET_JAPANESE) Sjis2Euc(ai->labels[i], 16);
-   		if (char_set==CHAR_SET_1250) Win2Lat(ai->labels[i], 16);
-   		if (char_set==CHAR_SET_1251) win1251_to_koi8(ai->labels[i], 16);
-   		if (char_set==CHAR_SET_1251_B) koi8_to_win1251(ai->labels[i], 16);
-	     }
-	   for (i = 0; i < 8; i++)
-	     if (ai->phoneLabels[i][0] != '\0') {
-   		if (char_set==CHAR_SET_JAPANESE) Sjis2Euc(ai->phoneLabels[i], 16);
-   		if (char_set==CHAR_SET_1250) Win2Lat(ai->phoneLabels[i], 16);
-   		if (char_set==CHAR_SET_1251) win1251_to_koi8(ai->phoneLabels[i], 16);
-   		if (char_set==CHAR_SET_1251_B) koi8_to_win1251(ai->phoneLabels[i], 16);
-	     }
+      int i;
+      for (i = 0; i < 16; i++)
+	if (ai->category.name[i][0] != '\0') {
+	   charset_p2j(ai->category.name[i], 16, char_set);
+	}
+      for (i = 0; i < 19 + 3; i++)
+	if (ai->labels[i][0] != '\0') {
+	   charset_p2j(ai->labels[i],16, char_set);
+	}
+      for (i = 0; i < 8; i++)
+	if (ai->phoneLabels[i][0] != '\0') {
+	   charset_p2j(ai->phoneLabels[i],16, char_set);
 	}
    }
 
@@ -329,7 +322,7 @@ int get_addresses2(AddressList **address_list, int sort_order,
    int keep_priv;
    long char_set;
    buf_rec *br;
-  
+
    jpilot_logf(LOG_DEBUG, "get_addresses2()\n");
    if (modified==2) {
       get_pref(PREF_SHOW_MODIFIED, &keep_modified, NULL);
@@ -385,17 +378,11 @@ int get_addresses2(AddressList **address_list, int sort_order,
       }
 
       get_pref(PREF_CHAR_SET, &char_set, NULL);
-      if (char_set==CHAR_SET_JAPANESE || 
-	  char_set==CHAR_SET_1250 ||
-	  char_set==CHAR_SET_1251 ||
-	  char_set==CHAR_SET_1251_B
-	  ) {
-	 int i;
+      if (char_set != CHAR_SET_ENGLISH) {
 	 for (i = 0; i < 19; i++) {
-	    if (char_set==CHAR_SET_JAPANESE) Sjis2Euc(a.entry[i], 65536);
-	    if (char_set==CHAR_SET_1250) Win2Lat(a.entry[i], 65536);
-	    if (char_set==CHAR_SET_1251) win1251_to_koi8(a.entry[i], 65536);
-	    if (char_set==CHAR_SET_1251_B) koi8_to_win1251(a.entry[i], 65536);
+	    if ((a.entry[i] != NULL) && (a.entry[i][0] != '\0')) {
+	       charset_p2j(a.entry[i], strlen(a.entry[i])+1, char_set);
+	    }
 	 }
       }
 
