@@ -1,6 +1,7 @@
 /* address.h
+ * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,5 +42,7 @@ int get_addresses2(AddressList **address_list, int sort_order,
 		  int modified, int deleted, int privates, int category);
 
 int address_print();
+int address_import(GtkWidget *window);
+int address_export(GtkWidget *window);
 
 #endif
