@@ -1,6 +1,7 @@
 /* alarms.c
+ * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 2000 by Judd Montgomery
+ * Copyright (C) 2000-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +47,13 @@
 
 /* #define ALARMS_DEBUG */
 
+
+/*
+ * Throughout the code, event_time is the time of the event
+ * alarm_time is the event_time - advance
+ * remind_time is the time a window is to be popped up (it may be postponed)
+ */
+
 typedef enum {
    ALARM_NONE = 0,
    ALARM_NEW,
@@ -56,20 +64,20 @@ typedef enum {
 struct jp_alarms {
    unsigned int unique_id;
    AlarmType type;
-   time_t alarm_time;
+   time_t event_time;
    time_t alarm_advance;
    struct jp_alarms *next;
 };
- 
+
 struct alarm_dialog_data {
    unsigned int unique_id;
-   time_t alarm_time;
+   time_t remind_time;
    GtkWidget *remind_entry;
    GtkWidget *radio1;
    GtkWidget *radio2;
    int button_hit;
 };
- 
+
 static struct jp_alarms *alarm_list=NULL;
 static struct jp_alarms *Plast_alarm_list=NULL;
 static struct jp_alarms *next_alarm=NULL;
@@ -127,20 +135,20 @@ static gboolean cb_destroy_dialog(GtkWidget *widget)
    if (Pdata->button_hit==DIALOG_SAID_2) {
       remind = atoi(entry);
       jpilot_logf(LOG_DEBUG, "remind entry = [%s]\n", entry);
-      set_pref_char(PREF_REMIND_IN, entry);
+      set_pref(PREF_REMIND_IN, 0, entry);
       if (GTK_TOGGLE_BUTTON(Pdata->radio1)->active) {
-	 set_pref(PREF_REMIND_UNITS, 0);
+	 set_pref(PREF_REMIND_UNITS, 0, NULL);
 	 remind *= 60;
       } else {
-	 set_pref(PREF_REMIND_UNITS, 1);
+	 set_pref(PREF_REMIND_UNITS, 1, NULL);
 	 remind *= 3600;
       }
       time(&ltime);
       localtime(&ltime);
-      advance = -(ltime + remind - Pdata->alarm_time);
+      advance = -(ltime + remind - Pdata->remind_time);
       alarms_add_to_list(Pdata->unique_id,
 			 ALARM_POSTPONED,
-			 Pdata->alarm_time,
+			 Pdata->remind_time,
 			 advance);
    }
    free(Pdata);
@@ -151,7 +159,7 @@ static gboolean cb_destroy_dialog(GtkWidget *widget)
 int dialog_alarm(char *title, char *frame_text,
 		 char *time_str, char *desc_str, char *note_str,
 		 unsigned int unique_id,
-		 time_t alarm_time)
+		 time_t remind_time)
 {
    GSList *group;
    GtkWidget *button, *label;
@@ -182,7 +190,7 @@ int dialog_alarm(char *title, char *frame_text,
    gtk_signal_connect(GTK_OBJECT(alarm_dialog), "destroy",
                       GTK_SIGNAL_FUNC(cb_destroy_dialog), alarm_dialog);
 
-   
+
    frame = gtk_frame_new(frame_text);
    gtk_frame_set_label_align(GTK_FRAME(frame), 0.5, 0.0);
    vbox1 = gtk_vbox_new(FALSE, 2);
@@ -191,7 +199,7 @@ int dialog_alarm(char *title, char *frame_text,
    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
    gtk_container_set_border_width(GTK_CONTAINER(vbox1), 5);
    gtk_container_set_border_width(GTK_CONTAINER(hbox1), 5);
-   
+
    gtk_container_add(GTK_CONTAINER(alarm_dialog), frame);
    gtk_container_add(GTK_CONTAINER(frame), vbox1);
 
@@ -244,7 +252,7 @@ int dialog_alarm(char *title, char *frame_text,
    Pdata = malloc(sizeof(struct alarm_dialog_data));
    if (Pdata) {
       Pdata->unique_id = unique_id;
-      Pdata->alarm_time = alarm_time;
+      Pdata->remind_time = remind_time;
       /* Set the default button pressed to OK */
       Pdata->button_hit = DIALOG_SAID_1;     
       Pdata->remind_entry=remind_entry;
@@ -267,7 +275,7 @@ int dialog_alarm(char *title, char *frame_text,
    gtk_box_pack_start(GTK_BOX(hbox1), vbox_temp, TRUE, TRUE, 1);
    gtk_box_pack_start(GTK_BOX(vbox_temp), radio1, TRUE, TRUE, 1);
    gtk_box_pack_start(GTK_BOX(vbox_temp), radio2, TRUE, TRUE, 1);
-   
+
    gtk_widget_show_all(alarm_dialog);
 
    return 0;
@@ -283,7 +291,7 @@ const char *print_date(const time_t t1)
    static char str[100];
 
    Pnow = localtime(&t1);
-   strftime(str, 80, "%B %d, %Y %H:%M", Pnow);
+   strftime(str, 80, "%B %d, %Y %H:%M:%S", Pnow);
    return str;
 }
 const char *print_type(AlarmType type)
@@ -302,11 +310,11 @@ const char *print_type(AlarmType type)
    }
 }
 #else
-const char *print_date(const time_t t1)
+inline const char *print_date(const time_t t1)
 {
    return "";
 }
-const char *print_type(AlarmType type)
+inline const char *print_type(AlarmType type)
 {
    return "";
 }
@@ -315,11 +323,11 @@ const char *print_type(AlarmType type)
 
 void alarms_add_to_list(unsigned int unique_id,
 			AlarmType type,
-			time_t alarm_time,
+			time_t event_time,
 			time_t alarm_advance)
 {
    struct jp_alarms *temp_alarm;
-   
+
 #ifdef ALARMS_DEBUG
    printf("alarms_add_to_list()\n");
 #endif
@@ -331,7 +339,7 @@ void alarms_add_to_list(unsigned int unique_id,
    }
    temp_alarm->unique_id = unique_id;
    temp_alarm->type = type;
-   temp_alarm->alarm_time = alarm_time;
+   temp_alarm->event_time = event_time;
    temp_alarm->alarm_advance = alarm_advance;
    temp_alarm->next = NULL;
    if (Plast_alarm_list) {
@@ -380,7 +388,7 @@ void alarms_remove_from_to_list(unsigned int unique_id)
 void free_alarms_list(int mask)
 {
    struct jp_alarms *ta, *ta_next;
-   
+
    if (mask&PREV_ALARM_MASK) {
       for (ta=alarm_list; ta; ta=ta_next) {
 	 ta_next=ta->next;
@@ -405,12 +413,12 @@ void alarms_write_file(void)
    int fail, n;
    time_t ltime;
    struct tm *now;
-   
+
    jpilot_logf(LOG_DEBUG, "alarms_write_file()\n");
 
    time(&ltime);
    now = localtime(&ltime);
-   
+
    out=jp_open_home_file("jpilot.alarms.tmp", "w");
    if (!out) {
       jpilot_logf(LOG_WARN, "Could not open jpilot.alarms.tmp file\n");
@@ -440,7 +448,7 @@ void alarms_write_file(void)
    if (n<1) fail=1;
 
    fclose(out);
-   
+
    if (fail) {
       unlink_file("jpilot.alarms.tmp");
    } else {
@@ -455,7 +463,7 @@ void make_command_safe(char *command)
 {
    int i, len;
    char c;
-   
+
    len = strlen(command);
    for (i=0; i<len; i++) {
       c=command[i];
@@ -492,7 +500,7 @@ int alarms_do_one(struct Appointment *a,
    const char *pref_command;
    char c1, c2;
    int i, len;
-   
+
    alarms_write_file();
 
    switch (type) {
@@ -533,8 +541,9 @@ int alarms_do_one(struct Appointment *a,
    get_pref(PREF_ALARM_COMMAND, &ivalue, &pref_command);
    get_pref(PREF_DO_ALARM_COMMAND, &do_command, NULL);
 #ifdef ALARMS_DEBUG
-   printf("pref_command = %s\n", pref_command);
-#endif   
+   printf("pref_command = [%s]\n", pref_command);
+#endif
+   bzero(command, 1024);
    if (do_command) {
       command[0]='\0';
       for (i=0; i<MAX_PREF_VALUE-1; i++) {
@@ -542,33 +551,38 @@ int alarms_do_one(struct Appointment *a,
 	 c2 = pref_command[i+1];
 	 len = strlen(command);
 	 /* expand '%t' */
-	 if ((c1=='%') && (c2=='t')) {
-	    i++;
-	    strncat(command, time1_str, 1022-len);
-	    continue;
-	 }
-	 /* expand '%d' */
-	 if ((c1=='%') && (c2=='d')) {
-	    i++;
-	    strncat(command, date_str, 1022-len);
-	    continue;
-	 }
+	 if (c1=='%') {
+	    if (c2=='t') {
+	       i++;
+	       strncat(command, time1_str, 1022-len);
+	       continue;
+	    }
+	    /* expand '%d' */
+	    if (c2=='d') {
+	       i++;
+	       strncat(command, date_str, 1022-len);
+	       continue;
+	    }
 #ifdef ALARM_SHELL_DESC_NOTE
-	 /* expand '%D' */
-	 if ((c1=='%') && (c2=='D')) {
-	    i++;
-	    strncat(command, desc_str, 1022-len);
-	    continue;
-	 }
-	 if ((c1=='%') && (c2=='N')) {
-	    i++;
-	    strncat(command, note_str, 1022-len);
-	    continue;
-	 }
+	    /* expand '%D' */
+	    if (c2=='D') {
+	       i++;
+	       strncat(command, desc_str, 1022-len);
+	       continue;
+	    }
+	    if (c2=='N') {
+	       i++;
+	       strncat(command, note_str, 1022-len);
+	       continue;
+	    }
 #endif
+	 }
 	 if (len<1020) {
 	    command[len++]=c1;
-	    command[len]='\0';;
+	    command[len]='\0';
+	 }
+	 if (c1=='\0') {
+	    break;
 	 }
       }
       command[1022]='\0';
@@ -603,7 +617,8 @@ gint cb_timer_alarms(gpointer data)
    time_t t, diff;
    time_t t_alarm_time;
    struct tm *Ptm;
-   
+   struct tm copy_tm;
+
    a_list=NULL;
 
    if (!first) {
@@ -612,13 +627,12 @@ gint cb_timer_alarms(gpointer data)
    }
 
    time(&t);
-   localtime(&t);
 
    for (temp_alarm=alarm_list; temp_alarm; temp_alarm=ta_next) {
       ta_next=temp_alarm->next;
-      diff = temp_alarm->alarm_time - t - temp_alarm->alarm_advance;
+      diff = temp_alarm->event_time - t - temp_alarm->alarm_advance;
       if (temp_alarm->type!=ALARM_MISSED) {
-	 if (diff > ALARM_INTERVAL/2) {
+	 if (diff >= ALARM_INTERVAL/2) {
 	    continue;
 	 }
       }
@@ -628,7 +642,7 @@ gint cb_timer_alarms(gpointer data)
 #ifdef ALARMS_DEBUG
       printf("unique_id=%d\n", temp_alarm->unique_id);
       printf("type=%s\n", print_type(temp_alarm->type));
-      printf("alarm_time=%s\n", print_date(temp_alarm->alarm_time));
+      printf("event_time=%s\n", print_date(temp_alarm->event_time));
       printf("alarm_advance=%ld\n", temp_alarm->alarm_advance);
 #endif
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
@@ -638,7 +652,7 @@ gint cb_timer_alarms(gpointer data)
 #endif
 	    alarms_do_one(&(temp_al->ma.a),
 			  temp_alarm->unique_id,
-			  temp_alarm->alarm_time,
+			  temp_alarm->event_time,
 			  ALARM_MISSED);
 	    break;
 	 }
@@ -649,8 +663,8 @@ gint cb_timer_alarms(gpointer data)
    }
 
    if (next_alarm) {
-      diff = next_alarm->alarm_time - t;
-      if (diff < ALARM_INTERVAL/2) {
+      diff = next_alarm->event_time - t - next_alarm->alarm_advance;
+      if (diff <= ALARM_INTERVAL/2) {
 	 if (a_list==NULL) {
 	    get_days_appointments2(&a_list, NULL, 0, 0, 1);
 	 }
@@ -660,26 +674,29 @@ gint cb_timer_alarms(gpointer data)
 #ifdef ALARMS_DEBUG
 		  printf("** next unique_id=%d\n", temp_alarm->unique_id);
 		  printf("** next type=%s\n", print_type(temp_alarm->type));
-		  printf("** next alarm_time=%s\n", print_date(temp_alarm->alarm_time));
+		  printf("** next event_time=%s\n", print_date(temp_alarm->event_time));
 		  printf("** next alarm_advance=%ld\n", temp_alarm->alarm_advance);
 		  printf("** next %s\n", temp_al->ma.a.description);
 #endif		  
 		  alarms_do_one(&(temp_al->ma.a),
 				temp_alarm->unique_id,
-				temp_alarm->alarm_time,
+				temp_alarm->event_time,
 				ALARM_NEW);
 		  break;
 	       }
 	    }
 	    /* This may not be exactly right */
-	    t_alarm_time = temp_alarm->alarm_time + temp_alarm->alarm_advance + 1;
+	    t_alarm_time = temp_alarm->event_time + 1;
+#ifdef ALARMS_DEBUG
+	    printf("** t_alarm_time-->%s\n", print_date(t_alarm_time));
+#endif
 	    ta_next=temp_alarm->next;
 	    free(temp_alarm);
 	    next_alarm = ta_next;
 	 }
-
 	 Ptm = localtime(&t_alarm_time);
-	 alarms_find_next(Ptm, Ptm, TRUE);
+	 memcpy(&copy_tm, Ptm, sizeof(struct tm));
+	 alarms_find_next(&copy_tm, &copy_tm, TRUE);
       }
    }
    if (a_list) {
@@ -724,14 +741,14 @@ static int find_prev_next(struct Appointment *a,
 #endif
    *prev_found=*next_found=0;
    forward=backward=1;
-   
+
    t1=mktime(date1);
    t2=mktime(date2);
 
    bzero(tm_prev, sizeof(struct tm));
    bzero(tm_next, sizeof(struct tm));
 
-   bzero(&t, sizeof(t));
+   bzero(&t, sizeof(struct tm));
    t.tm_year=a->begin.tm_year;
    t.tm_mon=a->begin.tm_mon;
    t.tm_mday=a->begin.tm_mday;
@@ -742,6 +759,9 @@ static int find_prev_next(struct Appointment *a,
    freq = 0;
    switch (a->repeatType) {
     case repeatNone:
+#ifdef ALARMS_DEBUG
+      printf("fpn: repeatNone\n");
+#endif
       t_alarm=mktime(&(a->begin)) - adv;
       if ((t_alarm < t2) && (t_alarm > t1)) {
 	 memcpy(tm_prev, &(a->begin), sizeof(struct tm));
@@ -771,7 +791,7 @@ static int find_prev_next(struct Appointment *a,
 	 t_future = (((t2 + adv - t_alarm) / t_interval) + 1) *t_interval + t_alarm;
 	 *prev_found=*next_found=1;
       } else {
-	 t_future = t_alarm - adv;
+	 t_future = t_alarm;
 	 *next_found=1;
       }
       Pnow = localtime(&t_past);
@@ -790,6 +810,9 @@ static int find_prev_next(struct Appointment *a,
 #endif
       break;
     case repeatWeekly:
+#ifdef ALARMS_DEBUG
+      printf("fpn: repeatWeekly\n");
+#endif
       freq = a->repeatFrequency;
       t.tm_year=date2->tm_year;
       t.tm_mon=date2->tm_mon;
@@ -839,6 +862,9 @@ static int find_prev_next(struct Appointment *a,
       }
       break;
     case repeatMonthlyByDay:
+#ifdef ALARMS_DEBUG
+      printf("fpn: repeatMonthlyByDay\n");
+#endif
       t.tm_mon=date2->tm_mon;
       t.tm_year=date2->tm_year;
       freq = a->repeatFrequency;
@@ -871,6 +897,9 @@ static int find_prev_next(struct Appointment *a,
 #endif
       break;
     case repeatMonthlyByDate:
+#ifdef ALARMS_DEBUG
+      printf("fpn: repeatMonthlyByDate\n");
+#endif
       t.tm_mon=date2->tm_mon;
       t.tm_year=date2->tm_year;
       freq = a->repeatFrequency;
@@ -884,6 +913,9 @@ static int find_prev_next(struct Appointment *a,
       }
       break;
     case repeatYearly:
+#ifdef ALARMS_DEBUG
+      printf("fpn: repeatYearly\n");
+#endif
       t.tm_year=date2->tm_year;
       freq = a->repeatFrequency;
       offset = (t.tm_year - a->begin.tm_year)%(a->repeatFrequency);
@@ -1051,7 +1083,7 @@ static int find_prev_next(struct Appointment *a,
    return 0;
 }
 
-	     
+
 /*
  * Find the next appointment alarm
  *  if soonest_only then return the next alarm, else return all alarms
@@ -1094,6 +1126,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    date1.tm_mday=tm_temp->tm_mday;
    date1.tm_hour=tm_temp->tm_hour;
    date1.tm_min=tm_temp->tm_min;
+   date1.tm_sec=tm_temp->tm_sec;
    date1.tm_isdst=tm_temp->tm_isdst;
 
    if (!date2_in) {
@@ -1108,9 +1141,10 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    date2.tm_mday=tm_temp->tm_mday;
    date2.tm_hour=tm_temp->tm_hour;
    date2.tm_min=tm_temp->tm_min;
+   date2.tm_sec=tm_temp->tm_sec;
    date2.tm_isdst=tm_temp->tm_isdst;
 
-   t1=mktime(&date1);	
+   t1=mktime(&date1);
    t2=mktime(&date2);
 
 #ifdef ALARMS_DEBUG
@@ -1126,13 +1160,13 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 	printf("^date1=%s\n", str);
      }
 #endif
-   
+
    if (!soonest_only) {
       free_alarms_list(PREV_ALARM_MASK | NEXT_ALARM_MASK);
    } else {
       free_alarms_list(NEXT_ALARM_MASK);
    }
-   
+
    a_list=NULL;
    get_days_appointments2(&a_list, NULL, 0, 0, 1);
 
@@ -1165,8 +1199,10 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       /* If the appointment has an end date, see that we are not past it */
       if (!(temp_al->ma.a.repeatForever)) {
 	 t_end = mktime(&(temp_al->ma.a.repeatEnd));
+	 /* We need to add 24 hours to the end date to make it inclusive */
+	 t_end += 86400;
 	 t_begin = mktime(&(temp_al->ma.a.begin));
-	 if (t_begin < t1) {
+	 if (t_end < t2) {
 #ifdef ALARMS_DEBUG      
 	    printf("past end date\n");
 #endif
@@ -1200,10 +1236,10 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 #ifdef ALARMS_DEBUG      
       printf("adv=%ld\n", adv);
 #endif
-      
+
       t_prev=t_future=t_interval=0;
       prev_found=next_found=0;
-      
+
       find_prev_next(&(temp_al->ma.a),
 		     adv,
 		     &date1,
@@ -1265,16 +1301,16 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 	 if (next_alarm==NULL) {
 	    add_a_next=1;
 	 } else if 
-	   ((t_future<next_alarm->alarm_time-next_alarm->alarm_advance) ||
-	    (t_future==next_alarm->alarm_time-next_alarm->alarm_advance)) {
-	    add_a_next=1;
-	      if (t_future<next_alarm->alarm_time-next_alarm->alarm_advance) {
+	   (t_future - adv <= next_alarm->event_time - next_alarm->alarm_advance) {
+	      add_a_next=1;
+	      if (t_future - adv < next_alarm->event_time - next_alarm->alarm_advance) {
 #ifdef ALARMS_DEBUG
-	       printf("freeing next alarms\n");
+		 printf("next alarm=%s\n", print_date(next_alarm->event_time - next_alarm->alarm_advance));
+		 printf("freeing next alarms\n");
 #endif
-	       free_alarms_list(NEXT_ALARM_MASK);
-	    }
-	 }
+		 free_alarms_list(NEXT_ALARM_MASK);
+	      }
+	   }
 	 if (add_a_next) {
 #ifdef ALARMS_DEBUG
 	    printf("found a new next\n");
@@ -1285,7 +1321,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 	       next_alarm = ta;
 	       next_alarm->unique_id = temp_al->ma.unique_id;
 	       next_alarm->type = ALARM_NEW;
-	       next_alarm->alarm_time = t_future;
+	       next_alarm->event_time = t_future;
 	       next_alarm->alarm_advance = adv;
 	    }
 	 }
@@ -1313,7 +1349,11 @@ int alarms_init(unsigned char skip_past_alarms,
    int year, mon, day, hour, min, n;
 
    jpilot_logf(LOG_DEBUG, "alarms_init()\n");
-   
+
+   alarm_list=NULL;
+   Plast_alarm_list=NULL;
+   next_alarm=NULL;
+
    total_alarm_windows = 0;
    glob_skip_all_alarms = skip_all_alarms;
 
@@ -1331,7 +1371,7 @@ int alarms_init(unsigned char skip_past_alarms,
       jpilot_logf(LOG_WARN, "Could not open jpilot.alarms file\n");
       return -1;
    }
-   
+
    while (!feof(in)) {
       line[0]='\0';
       fgets(line, 255, in);
@@ -1375,7 +1415,7 @@ int alarms_init(unsigned char skip_past_alarms,
    tm1.tm_isdst=-1;
 
    mktime(&tm1);
-   
+
    alarms_find_next(&tm1, &now, FALSE);
 
    gtk_timeout_add(ALARM_INTERVAL*1000, cb_timer_alarms, NULL);
