@@ -1,11 +1,11 @@
 /* jpilot.c
+ * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,10 +51,13 @@
 #include "alarms.h"
 #endif
 #include "print.h"
+#include "restore.h"
 #include "password.h"
 #include "i18n.h"
 
-#ifndef WITH_SYMPHONET
+#include "icons/jpilot-icon4.xpm"
+
+#ifndef WITH_PROMETHEON
 #include "datebook.xpm"
 #include "address.xpm"
 #include "todo.xpm"
@@ -108,7 +111,8 @@ int glob_app = 0;
 int glob_focus = 1;
 GtkWidget *glob_dialog=NULL;
 unsigned char skip_plugins;
-				       
+static GtkWidget *box_locked, *box_locked_masked, *box_unlocked;
+
 int pipe_in, pipe_out;
 
 GtkWidget *sync_window = NULL;
@@ -222,7 +226,7 @@ gboolean parse_geometry(const char *str,
 static void cb_focus(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
    int i;
-   
+
    i = GPOINTER_TO_INT(data);
    if (i==0) {
       glob_focus=0;
@@ -250,8 +254,8 @@ int gui_cleanup()
 #ifdef ENABLE_PLUGINS
    struct plugin_s *plugin;
    GList *plugin_list, *temp_list;
-#endif   
-   
+#endif
+
 #ifdef ENABLE_PLUGINS
    plugin_list = NULL;
    plugin_list = get_plugin_list();
@@ -269,7 +273,7 @@ int gui_cleanup()
       }
    }
 #endif   
-   
+
    switch(glob_app) {
     case DATEBOOK:
       datebook_gui_cleanup();
@@ -294,11 +298,11 @@ void call_plugin_gui(int number, int unique_id)
 {
    struct plugin_s *plugin;
    GList *plugin_list, *temp_list;
-   
+
    if (!number) {
       return;
    }
-   
+
    gui_cleanup();
 
    plugin_list = NULL;
@@ -332,7 +336,7 @@ void cb_plugin_gui(GtkWidget *widget, int number)
 {
    call_plugin_gui(number, 0);
 }
-      
+
 #endif
 
 #ifdef ENABLE_PLUGINS
@@ -340,16 +344,15 @@ void call_plugin_help(int number)
 {
    struct plugin_s *plugin;
    GList *plugin_list, *temp_list;
-   char *button_text[]={"OK"
+   char *button_text[]={gettext_noop("OK")
    };
    char *text;
-   char temp[256];
    int width, height;
 
    if (!number) {
       return;
    }
-   
+
    plugin_list = NULL;
    plugin_list = get_plugin_list();
 
@@ -359,14 +362,10 @@ void call_plugin_help(int number)
       plugin = (struct plugin_s *)temp_list->data;
       if (plugin) {
 	 if (plugin->number == number) {
-	    glob_app = plugin->number;
 	    if (plugin->plugin_help) {
 	       text = NULL;
 	       plugin->plugin_help(&text, &width, &height);
 	       if (text) {
-		  strncpy(temp, _("OK"), 256);
-		  temp[255]='\0';
-		  button_text[0]=temp;
 		  dialog_generic(GTK_WIDGET(window)->window,
 				width, height,
 				 _("Help"), plugin->name, text, 1, button_text);
@@ -391,26 +390,25 @@ void cb_print(GtkWidget *widget, gpointer data)
    struct plugin_s *plugin;
    GList *plugin_list, *temp_list;
    char *button_text[]={gettext_noop("OK")};
-   char temp[256];
-   
+
    switch(glob_app) {
     case DATEBOOK:
-      if (print_gui(window) == DIALOG_SAID_PRINT) {
-	 datebook_print();
+      if (print_gui(window, DATEBOOK, 1, 0x07) == DIALOG_SAID_PRINT) {
+	 datebook_print(print_day_week_month);
       }
       return;
     case ADDRESS:
-      if (print_gui(window) == DIALOG_SAID_PRINT) {
+      if (print_gui(window, ADDRESS, 0, 0x00) == DIALOG_SAID_PRINT) {
 	 address_print();
       }
       return;
     case TODO:
-      if (print_gui(window) == DIALOG_SAID_PRINT) {
+      if (print_gui(window, TODO, 0, 0x00) == DIALOG_SAID_PRINT) {
 	 todo_print();
       }
       return;
     case MEMO:
-      if (print_gui(window) == DIALOG_SAID_PRINT) {
+      if (print_gui(window, MEMO, 0, 0x00) == DIALOG_SAID_PRINT) {
 	 memo_print();
       }
       return;
@@ -431,15 +429,112 @@ void cb_print(GtkWidget *widget, gpointer data)
       }
    }
 #endif
-   strncpy(temp, _("OK"), 256);
-   temp[255]='\0';
-   button_text[0]=temp;
    dialog_generic(GTK_WIDGET(window)->window, 0, 0,
 		  _("Print"), "", 
 		  _("There is no print support for this conduit."),
 		  1, button_text);
 }
 
+void cb_restore(GtkWidget *widget, gpointer data)
+{
+   int r;
+   int w, h, x, y;
+
+   jpilot_logf(LOG_DEBUG, "cb_restore()\n");
+
+   gdk_window_get_size(window->window, &w, &h);
+   gdk_window_get_root_origin(window->window, &x, &y);
+
+   w = w/2;
+   x+=40;
+
+   r = restore_gui(w, h, x, y);
+}
+
+void cb_import(GtkWidget *widget, gpointer data)
+{
+   struct plugin_s *plugin;
+   GList *plugin_list, *temp_list;
+   char *button_text[]={gettext_noop("OK")};
+
+   switch(glob_app) {
+    case DATEBOOK:
+      datebook_import(window);
+      return;
+    case ADDRESS:
+      address_import(window);
+      return;
+    case TODO:
+      todo_import(window);
+      return;
+    case MEMO:
+      memo_import(window);
+      return;
+   }
+#ifdef ENABLE_PLUGINS
+   plugin_list = NULL;
+   plugin_list = get_plugin_list();
+
+   for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
+      plugin = (struct plugin_s *)temp_list->data;
+      if (plugin) {
+	 if (glob_app == plugin->number) {
+	    if (plugin->plugin_import) {
+	       plugin->plugin_import(window);
+	       return;
+	    }
+	 }
+      }
+   }
+#endif
+   dialog_generic(GTK_WIDGET(window)->window, 0, 0,
+		  _("Import"), "", 
+		  _("There is no import support for this conduit."),
+		  1, button_text);
+}
+
+void cb_export(GtkWidget *widget, gpointer data)
+{
+   struct plugin_s *plugin;
+   GList *plugin_list, *temp_list;
+   char *button_text[]={gettext_noop("OK")};
+
+   switch(glob_app) {
+    case DATEBOOK:
+      datebook_export(window);
+      return;
+    case ADDRESS:
+      address_export(window);
+      return;
+    case TODO:
+      todo_export(window);
+      return;
+    case MEMO:
+      memo_export(window);
+      return;
+   }
+#ifdef ENABLE_PLUGINS
+   plugin_list = NULL;
+   plugin_list = get_plugin_list();
+
+   for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
+      plugin = (struct plugin_s *)temp_list->data;
+      if (plugin) {
+	 if (glob_app == plugin->number) {
+	    if (plugin->plugin_export) {
+	       plugin->plugin_export(window);
+	       return;
+	    }
+	 }
+      }
+   }
+#endif
+   dialog_generic(GTK_WIDGET(window)->window, 0, 0,
+		  _("Export"), "", 
+		  _("There is no export support for this conduit."),
+		  1, button_text);
+}
+
 static void cb_private(GtkWidget *widget, gpointer data)
 {
    int privates, was_privates;
@@ -448,26 +543,43 @@ static void cb_private(GtkWidget *widget, gpointer data)
 
    was_privates = privates = show_privates(GET_PRIVATES, NULL);
 
-   if (privates==SHOW_PRIVATES) {
+   switch (privates) {
+    case SHOW_PRIVATES:
+      privates = show_privates(MASK_PRIVATES, NULL);
+      gtk_widget_hide(box_locked);
+      gtk_widget_show(box_locked_masked);
+      gtk_widget_hide(box_unlocked);
+      break;
+    case MASK_PRIVATES:
       privates = show_privates(HIDE_PRIVATES, NULL);
-   } else {
+      gtk_widget_show(box_locked);
+      gtk_widget_hide(box_locked_masked);
+      gtk_widget_hide(box_unlocked);
+      break;
+    case HIDE_PRIVATES:
       ret = dialog_password(ascii_password);
       if (ret==1) {
 	 privates = show_privates(SHOW_PRIVATES, ascii_password);
+	 if (privates==SHOW_PRIVATES) {
+	    gtk_widget_hide(box_locked);
+	    gtk_widget_hide(box_locked_masked);
+	    gtk_widget_show(box_unlocked);
+	 }
       }
+      break;
    }
 
    if (was_privates!=privates) {
       cb_app_button(NULL, GINT_TO_POINTER(REDRAW));
    }
 }
-   
+
 void cb_app_button(GtkWidget *widget, gpointer data)
 {
    int app;
-   
+
    app = GPOINTER_TO_INT(data);
-   
+
    if (app==REDRAW) {
       gui_cleanup();
       if (glob_date_timer_tag) {
@@ -479,7 +591,7 @@ void cb_app_button(GtkWidget *widget, gpointer data)
       app = glob_app;
       glob_app = 0;
    }
-   
+
    switch(app) {
     case DATEBOOK:
       if (glob_app == DATEBOOK) {
@@ -568,11 +680,10 @@ void cb_app_button(GtkWidget *widget, gpointer data)
    }
 }
 
-void cb_sync_hide(GtkWidget *widget, gpointer data)
+void cb_sync(GtkWidget *widget, unsigned int flags)
 {
-   gtk_widget_destroy(sync_window);
-   
-   sync_window=NULL;
+   setup_sync(flags);
+   return;
 }
 
 /*
@@ -586,21 +697,20 @@ void bad_sync_exit_status(int exit_status)
      /*-------------------------------------------*/
      "This palm doesn't have the same user name\n"
      "or user ID as the one that was synced the\n"
-     "last time.  Syncing could have unwanted\n"
-     "effects.\n"
+     "last time.  Syncing could have unwanted effects."
+     "\n"
      "Read the user manual if you are uncertain.";
    char text2[] =
      /*-------------------------------------------*/
      "This palm has a NULL user id.\n"
-     "It may have been hard reset.\n"
-     "J-Pilot will not restore a palm yet.\n"
-     "Use pilot-xfer to restore the palm and\n"
-     "install-user to add a username and user ID\n"
-     "to the palm.\n"
+     "Every palm must have a unique user id in order to sync properly.\n"
+     "If it has been hard reset, use restore from the menu to restore it,\n"
+     "or use pilot-xfer.\n"
+     "To add a user name and ID use install-user, i.e install-user \"name\" 12345.\n"
      "Read the user manual if you are uncertain.";
-   char *button_text[]={"OK", "Sync Anyway"
+   char *button_text[]={gettext_noop("OK"), gettext_noop("Sync Anyway")
    };
-   
+
    if (!GTK_IS_WINDOW(window)) {
       return;
    }
@@ -674,7 +784,7 @@ void cb_read_pipe(gpointer data,
 	       strncpy(user, Pstr2, user_len);
 	       user[user_len] = '\0';
 	       jpilot_logf(LOG_DEBUG, "pipe_read: user = %s\n", user);
-	       set_pref_char(PREF_USER, user);
+	       set_pref(PREF_USER, 0, user);
 	    }
 	 }
       }
@@ -697,7 +807,7 @@ void cb_read_pipe(gpointer data,
 	       memmove(Pstr1, Pstr3+1, buf_len);
 	       password[password_len] = '\0';
 	       jpilot_logf(LOG_DEBUG, "pipe_read: password = %s\n", password);
-	       set_pref_char(PREF_PASSWORD, password);
+	       set_pref(PREF_PASSWORD, 0, password);
 	    }
 	 }
       }
@@ -709,7 +819,7 @@ void cb_read_pipe(gpointer data,
 	 num = sscanf(Pstr2, "%ld", &user_id);
 	 if (num > 0) {
 	    jpilot_logf(LOG_DEBUG, "pipe_read: user id = %ld\n", user_id);
-	    set_pref(PREF_USER_ID, user_id);
+	    set_pref(PREF_USER_ID, user_id, NULL);
 	 } else {
 	    jpilot_logf(LOG_DEBUG, "pipe_read: trouble reading user id\n");
 	 }
@@ -753,37 +863,48 @@ void cb_read_pipe(gpointer data,
 void cb_about(GtkWidget *widget, gpointer data)
 {
    char text[255];
-   char *button_text[1];
-   char temp[256];
+   char *button_text[]={gettext_noop("OK")};
    char about[256];
 
    sprintf(text,
 	   /*-------------------------------------------*/
 	   _("%s was written by\n"
-	     "Judd Montgomery (c) 1999-2000.\n"
+	     "Judd Montgomery (c) 1999-2001.\n"
 	     "judd@jpilot.org\n"
 	     "http://jpilot.org\n"),   
 	   PN);
    g_snprintf(about, 250, _("About %s"), PN);
 
    if (GTK_IS_WINDOW(window)) {
-      strncpy(temp, _("OK"), 256);
-      temp[255]='\0';
-      button_text[0]=temp;
       dialog_generic(GTK_WIDGET(window)->window,
  		     0, 0,
 		     about, "oOo", text, 1, button_text);
    }
 }
 
+void cb_install_gui(GtkWidget *widget, gpointer data)
+{
+   int w, h, x, y;
+
+   jpilot_logf(LOG_DEBUG, "cb_install_gui()\n");
+
+   gdk_window_get_size(window->window, &w, &h);
+   gdk_window_get_root_origin(window->window, &x, &y);
+
+   w = w/2;
+   x+=40;
+
+   install_gui(w, h, x, y);
+}
+
 void get_main_menu(GtkWidget  *window,
 		   GtkWidget **menubar,
 		   GList *plugin_list)
 /* Some of this code was copied from the gtk_tut.txt file */
-#ifndef WITH_SYMPHONET
-#define NUM_FACTORY_ITEMS 19
+#ifndef WITH_PROMETHEON
+#define NUM_FACTORY_ITEMS 23
 #else
-#define NUM_FACTORY_ITEMS 17
+#define NUM_FACTORY_ITEMS 23
 #endif
 {
   GtkItemFactoryEntry menu_items1[NUM_FACTORY_ITEMS]={
@@ -792,12 +913,16 @@ void get_main_menu(GtkWidget  *window,
   { NULL, "<control>F", cb_search_gui,  0,        NULL },
   { NULL, NULL,         NULL,           0,        "<Separator>" },
   { NULL, "<control>I", cb_install_gui, 0,        NULL },
-  { NULL, "<control>E", cb_prefs_gui,   GPOINTER_TO_INT(window),   NULL },
+  { NULL, NULL, cb_import,              0,        NULL },
+  { NULL, "<control>A", cb_export,      0,        NULL },
+  { NULL, "<control>E", cb_prefs_gui,   0,        NULL },
   { NULL, "<control>P", cb_print,       0,        NULL },
-#ifndef WITH_SYMPHONET
+  { NULL, NULL,         NULL,           0,        "<Separator>" },
+  { NULL, NULL,         cb_restore,     0,        NULL },
+/* #ifndef WITH_PROMETHEON */
   { NULL, NULL,         NULL,           0,        "<Separator>" },
   { NULL, "<control>Q", delete_event,   0,        NULL },
-#endif
+/* #endif */
   { NULL, NULL,         NULL,           0,        "<Branch>" },
   { NULL, "<control>Z", cb_private,     0,        NULL },
   { NULL, "F1",         cb_app_button,  DATEBOOK, NULL },
@@ -828,18 +953,29 @@ void get_main_menu(GtkWidget  *window,
    int f_key_count;
 #endif
 
+   /* Irix doesn't like non-constant expressions in a static initializer */
+   /* So we have to do this to keep the compiler happy */
+   for (i=0; i<NUM_FACTORY_ITEMS; i++) {
+      if (menu_items1[i].callback==cb_prefs_gui) {
+	 menu_items1[i].callback_action = GPOINTER_TO_INT(window);
+	 break;
+      }
+   }
+
    i=0;
    menu_items1[i++].path=strdup(_("/File"));
    menu_items1[i++].path=strdup(_("/File/tear"));
    menu_items1[i++].path=strdup(_("/File/_Find"));
    menu_items1[i++].path=strdup(_("/File/sep1"));
    menu_items1[i++].path=strdup(_("/File/_Install"));
+   menu_items1[i++].path=strdup(_("/File/Import"));
+   menu_items1[i++].path=strdup(_("/File/Export"));
    menu_items1[i++].path=strdup(_("/File/Preferences"));
    menu_items1[i++].path=strdup(_("/File/_Print"));
-#ifndef WITH_SYMPHONET
+   menu_items1[i++].path=strdup(_("/File/sep1"));
+   menu_items1[i++].path=strdup(_("/File/Restore Handheld"));
    menu_items1[i++].path=strdup(_("/File/sep1"));
    menu_items1[i++].path=strdup(_("/File/Quit"));
-#endif
    menu_items1[i++].path=strdup(_("/_View"));
    menu_items1[i++].path=strdup(_("/View/Hide-Show Private Records"));
    menu_items1[i++].path=strdup(_("/View/Datebook"));
@@ -851,12 +987,12 @@ void get_main_menu(GtkWidget  *window,
    g_snprintf(temp_str, 100, _("/_Help/%s"), PN);
    temp_str[100]='\0';
    menu_items1[i++].path=strdup(temp_str);
-   
+
 #ifdef ENABLE_PLUGINS
    /* Go to first entry in the list */
    for (temp_list = plugin_list; temp_list; temp_list = temp_list->prev)
       plugin_list = temp_list;
-   
+
    /* Count the plugin/ entries */
    for (count=0, temp_list = plugin_list;
 	temp_list; temp_list = temp_list->next) {
@@ -865,7 +1001,7 @@ void get_main_menu(GtkWidget  *window,
 	 count++;
       }
    }
-   
+
    /* Count the help/ entries */
    for (help_count=0, temp_list = plugin_list;
 	temp_list; temp_list = temp_list->next) {
@@ -874,7 +1010,7 @@ void get_main_menu(GtkWidget  *window,
 	 help_count++;
       }
    }
-   
+
    plugin_menu_strings = plugin_help_strings = NULL;
    if (count != 0) {
      plugin_menu_strings = malloc(count * sizeof(char *));
@@ -882,7 +1018,7 @@ void get_main_menu(GtkWidget  *window,
    if (help_count != 0) {
       plugin_help_strings = malloc(help_count * sizeof(char *));
    }
-   
+
    /* Create plugin menu strings */
    str_i = 0;
    for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
@@ -913,7 +1049,7 @@ void get_main_menu(GtkWidget  *window,
    }
    nmenu_items = nmenu_items + help_count;
 #endif
-   
+
 
    menu_items2=malloc(nmenu_items * sizeof(GtkItemFactoryEntry));
    if (!menu_items2) {
@@ -939,6 +1075,7 @@ void get_main_menu(GtkWidget  *window,
 	   temp_list = temp_list->next, f_key_count++) {
 	 p = (struct plugin_s *)temp_list->data;
 	 if (!p->menu_name) {
+	    f_key_count--;
 	    continue;
 	 }
 	 menu_items2[i2].path=plugin_menu_strings[str_i];
@@ -961,7 +1098,7 @@ void get_main_menu(GtkWidget  *window,
    /* Skip the /Plugins entry */
    i1++;
 #endif
-   
+
    /* Copy the last part of the array until END */
    for (; ; i1++, i2++) {
       if (!strcmp(menu_items1[i1].path, "END")) {
@@ -1007,14 +1144,14 @@ void get_main_menu(GtkWidget  *window,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
    gtk_item_factory_create_items(item_factory, nmenu_items, menu_items2, NULL);
-   
+
    /* Attach the new accelerator group to the window. */
    gtk_accel_group_attach(accel_group, GTK_OBJECT (window));
 
    if (menubar)
      /* Finally, return the actual menu bar created by the item factory. */
      *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
-   
+
    free(menu_items2);
 
    /* NUM_FACTORY_ITEMS is just a safety, the loop stops at END */
@@ -1047,13 +1184,13 @@ static void delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
    /* gdk_window_get_deskrelative_origin(window->window, &x, &y); */
    gdk_window_get_origin(window->window, &x, &y);
    jpilot_logf(LOG_DEBUG, "x=%d, y=%d\n", x, y);
-   
+
    gdk_window_get_size(window->window, &pw, &ph);
-   set_pref(PREF_WINDOW_WIDTH, pw);
-   set_pref(PREF_WINDOW_HEIGHT, ph);
-   
-   set_pref(PREF_LAST_APP, glob_app);
-   
+   set_pref(PREF_WINDOW_WIDTH, pw, NULL);
+   set_pref(PREF_WINDOW_HEIGHT, ph, NULL);
+
+   set_pref(PREF_LAST_APP, glob_app, NULL);
+
    gui_cleanup();
 
 #ifdef ENABLE_PLUGINS
@@ -1069,12 +1206,13 @@ static void delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
       }
    }
 #endif
-   
+
    if (glob_child_pid) {
       jpilot_logf(LOG_DEBUG, "killing %d\n", glob_child_pid);
 	 kill(glob_child_pid, SIGTERM);
    }
-   write_rc_file();  /*jpilot.rc */
+   /* Write the jpilot.rc file from preferences */
+   pref_write_rc_file();
 
    cleanup_pc_files();
 
@@ -1086,14 +1224,14 @@ void cb_output(GtkWidget *widget, gpointer data)
    int flags;
    int w, h, output_height, pane_y;
    long ivalue;
-   
+
    flags=GPOINTER_TO_INT(data);
 
    if ((flags==OUTPUT_MINIMIZE) || (flags==OUTPUT_RESIZE)) {
       jpilot_logf(LOG_DEBUG,"paned pos = %d\n", GTK_PANED(output_pane)->handle_ypos);
       gdk_window_get_size(window->window, &w, &h);
       output_height = h - GTK_PANED(output_pane)->handle_ypos;
-      set_pref(PREF_OUTPUT_HEIGHT, output_height);
+      set_pref(PREF_OUTPUT_HEIGHT, output_height, NULL);
       if (flags==OUTPUT_MINIMIZE) {
 	 gtk_paned_set_position(GTK_PANED(output_pane), h + 2);
       }
@@ -1134,7 +1272,7 @@ void SetFontRecursively(GtkWidget *widget, gpointer data)
 {
    GtkStyle *style;
    GdkFont *f;
-   
+
    f = (GdkFont *)data;
 
    style = gtk_widget_get_style(widget);
@@ -1146,7 +1284,7 @@ void SetFontRecursively(GtkWidget *widget, gpointer data)
 }
 #endif
 
-int main(int   argc,
+int main(int argc,
 	 char *argv[])
 {
    GtkWidget *main_vbox;
@@ -1183,6 +1321,83 @@ int main(int   argc,
 #ifdef FONT_TEST
    GdkFont *f;
 #endif   
+char *xpm_locked[] = {
+   "15 18 4 1",
+   "       c None",
+   "b      c #000000000000",
+   "g      c #777777777777",
+   "w      c #FFFFFFFFFFFF",
+   "               ",
+   "      bbb      ",
+   "    bwwwwwb    ",
+   "   bwbbbbwbb   ",
+   "  bwb     bwb  ",
+   "  bwb     bwb  ",
+   "  bwb     bwb  ",
+   "  bwb     bwb  ",
+   " bbbbbbbbbbbbb ",
+   " bwwwwwwwwwwwb ",
+   " bgggggggggggb ",
+   " bwgwwwwwwwgwb ",
+   " bggwwwwwwwggb ",
+   " bwgwwwwwwwgwb ",
+   " bgggggggggggb ",
+   " bwwwwwwwwwwwb ",
+   " bbbbbbbbbbbbb ",
+   "               "
+};
+
+char *xpm_locked_masked[] = {
+   "15 18 4 1",
+   "       c None",
+   "b      c #000000000000",
+   "g      c #777777777777",
+   "w      c #FFFFFFFFFFFF",
+   "               ",
+   "      bbb      ",
+   "    bwwwwwb    ",
+   "   bwbbbbwbb   ",
+   "  bwb     bwb  ",
+   "  bwb     bwb  ",
+   "  bwb     bwb  ",
+   "  bwb     bwb  ",
+   " bbbbbbbbbbbbb ",
+   " bwwbbwwbbwwbb ",
+   " bbbwwbbwwbbwb ",
+   " bwwbbwwbbwwbb ",
+   " bbbwwbbwwbbwb ",
+   " bwwbbwwbbwwbb ",
+   " bbbwwbbwwbbwb ",
+   " bwwbbwwbbwwbb ",
+   " bbbbbbbbbbbbb ",
+   "               "
+};
+
+char *xpm_unlocked[] = {
+   "21 18 4 1",
+   "       c None",
+   "b      c #000000000000",
+   "g      c #777777777777",
+   "w      c #FFFFFFFF0000",
+   "                      ",
+   "              bbb     ",
+   "            bwwwwwb   ",
+   "           bwbbbbwbb  ",
+   "          bwb     bwb ",
+   "          bwb     bwb ",
+   "          bwb     bwb ",
+   "          bwb     bwb ",
+   " bbbbbbbbbbbbb        ",
+   " bwwwwwwwwwwwb        ",
+   " bgggggggggggb        ",
+   " bwwwwwwwwwwwb        ",
+   " bgggggggggggb        ",
+   " bwwwwwwwwwwwb        ",
+   " bgggggggggggb        ",
+   " bwwwwwwwwwwwb        ",
+   " bbbbbbbbbbbbb        ",
+   "                      "
+};
 
    sync_only=FALSE;
    skip_plugins=FALSE;
@@ -1194,7 +1409,9 @@ int main(int   argc,
    glob_log_gui_mask = LOG_FATAL | LOG_WARN | LOG_GUI;
    glob_find_id = 0;
 
-   read_rc_file();  /*jpilot.rc */
+   pref_init();
+   /* read jpilot.rc file for preferences */
+   pref_read_rc_file();
 
    w = h = x = y = bit_mask = 0;
 
@@ -1220,11 +1437,11 @@ int main(int   argc,
 	 skip_plugins = TRUE;
 	 jpilot_logf(LOG_INFO, "Not loading plugins.\n");
       }
-      if (!strncasecmp(argv[i], "-A", 2)) {
+      if (!strncmp(argv[i], "-A", 2)) {
 	 skip_all_alarms = TRUE;
 	 jpilot_logf(LOG_INFO, "Ignoring all alarms.\n");
       }
-      if (!strncasecmp(argv[i], "-a", 2)) {
+      if (!strncmp(argv[i], "-a", 2)) {
 	 skip_past_alarms = TRUE;
 	 jpilot_logf(LOG_INFO, "Ignoring past alarms.\n");
       }
@@ -1262,7 +1479,7 @@ int main(int   argc,
       jpilot_logf(LOG_DEBUG, "plugin: [%s] was loaded\n", plugin->name);
    }
 
-   
+
    for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
       plugin = (struct plugin_s *)temp_list->data;
       if (plugin) {
@@ -1288,7 +1505,7 @@ int main(int   argc,
    }
    pipe_in = filedesc[0];
    pipe_out = filedesc[1];
-   
+
    gtk_set_locale();
 
 #if defined(ENABLE_NLS)
@@ -1332,13 +1549,13 @@ int main(int   argc,
    gtk_init(&argc, &argv);
 
    read_gtkrc_file();
-   
+
    /* gtk_init must already have been called, or will seg fault */
    parse_geometry(geometry_str, pref_width, pref_height,
 		  &w, &h, &x, &y, &bit_mask);
 
    get_pref(PREF_USER, &ivalue, &svalue);
-   
+
    strcpy(title, PN" "VERSION);
    if ((svalue) && (svalue[0])) {
       strcat(title, " User: ");
@@ -1403,11 +1620,11 @@ int main(int   argc,
    gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, FALSE, 0);
    gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menubar), GTK_SHADOW_NONE);
 
-   
+
    gtk_box_pack_start(GTK_BOX(main_vbox), g_hbox, TRUE, TRUE, 3);
    gtk_container_set_border_width(GTK_CONTAINER(g_hbox), 10);
    gtk_box_pack_start(GTK_BOX(g_hbox), g_vbox0, FALSE, FALSE, 3);
-   
+
 
    /* Make the output text window */
    temp_hbox = gtk_hbox_new(FALSE, 0);
@@ -1449,7 +1666,7 @@ int main(int   argc,
    gtk_widget_set_usize(button_datebook, 46, 46);
    gtk_box_pack_start(GTK_BOX(g_vbox0), temp_hbox, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(temp_hbox), button_datebook, FALSE, FALSE, 0);
-   
+
    /* Create "Address" button */
    temp_hbox = gtk_hbox_new(FALSE, 0);
    button_address = gtk_button_new();
@@ -1487,14 +1704,22 @@ int main(int   argc,
 
    /* Create tooltips */
    glob_tooltips = gtk_tooltips_new();
-   
-#ifndef WITH_SYMPHONET
+
+   /* Create Lock/Unlock boxes */
+   temp_hbox = gtk_hbox_new(FALSE, 0);
+   box_locked = gtk_hbox_new(FALSE, 0);
+   box_locked_masked = gtk_hbox_new(FALSE, 0);
+   box_unlocked = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(g_vbox0), temp_hbox, FALSE, FALSE, 5);
+   gtk_box_pack_start(GTK_BOX(temp_hbox), box_locked, FALSE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(temp_hbox), box_locked_masked, FALSE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(temp_hbox), box_unlocked, FALSE, TRUE, 0);
+
    /* Create "Quit" button */
    button = gtk_button_new_with_label(_("Quit!"));
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(delete_event), NULL);
    gtk_box_pack_start(GTK_BOX(g_vbox0), button, FALSE, FALSE, 0);
-#endif
 
    /* Create "Sync" button */
    button = gtk_button_new_with_label(_("Sync"));
@@ -1527,11 +1752,15 @@ int main(int   argc,
    create_main_boxes();
 
    gtk_widget_show_all(window);
-   
+
    gtk_widget_show(window);
-   
+
    style = gtk_widget_get_style(window);
 
+   pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask, NULL, jpilot_icon4_xpm);
+   gdk_window_set_icon(window->window, NULL, pixmap, mask);
+   gdk_window_set_icon_name(window->window, PN);  
+
    /* Create "Datebook" pixmap */
    pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask,
 					 &style->bg[GTK_STATE_NORMAL],
@@ -1564,6 +1793,35 @@ int main(int   argc,
    gtk_widget_show(pixmapwid);
    gtk_container_add(GTK_CONTAINER(button_memo), pixmapwid);
 
+   /* Show locked box */
+   gtk_widget_show(box_locked);
+   gtk_widget_hide(box_locked_masked);
+   gtk_widget_hide(box_unlocked);
+
+   /* Create "locked" pixmap */
+   pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask,
+					 &style->bg[GTK_STATE_NORMAL],
+					 xpm_locked);
+   pixmapwid = gtk_pixmap_new(pixmap, mask);
+   gtk_widget_show(pixmapwid);
+   gtk_container_add(GTK_CONTAINER(box_locked), pixmapwid);
+
+   /* Create "locked/masked" pixmap */
+   pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask,
+					 &style->bg[GTK_STATE_NORMAL],
+					 xpm_locked_masked);
+   pixmapwid = gtk_pixmap_new(pixmap, mask);
+   gtk_widget_show(pixmapwid);
+   gtk_container_add(GTK_CONTAINER(box_locked_masked), pixmapwid);
+
+   /* Create "unlocked" pixmap */
+   pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask,
+					 &style->bg[GTK_STATE_NORMAL],
+					 xpm_unlocked);
+   pixmapwid = gtk_pixmap_new(pixmap, mask);
+   gtk_widget_show(pixmapwid);
+   gtk_container_add(GTK_CONTAINER(box_unlocked), pixmapwid);
+
    gtk_tooltips_set_tip(glob_tooltips, button_datebook, _("Datebook/Go to Today"), NULL);
 
    gtk_tooltips_set_tip(glob_tooltips, button_address, _("Address Book"), NULL);
