@@ -1,11 +1,11 @@
 /* libplugin.c
+ * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +41,7 @@ const char *jp_strstr(const char *haystack, const char *needle, int case_sense)
    register char *Ps2;
    register const char *Ps1;
    char *r;
-   
+
    if (case_sense) {
       return strstr(haystack, needle);
    } else {
@@ -53,7 +53,7 @@ const char *jp_strstr(const char *haystack, const char *needle, int case_sense)
       }
       needle2 = malloc(strlen(needle)+2);
       haystack2 = malloc(strlen(haystack)+2);
-      
+
       Ps1 = needle;
       Ps2 = needle2;
       while (Ps1[0]) {
@@ -62,7 +62,7 @@ const char *jp_strstr(const char *haystack, const char *needle, int case_sense)
 	 Ps2++;
       }
       Ps2[0]='\0';
-      
+
       Ps1 = haystack;
       Ps2 = haystack2;
       while (Ps1[0]) {
@@ -151,7 +151,7 @@ static int unpack_header(PC3RecordHeader *header, unsigned char *packed_header)
     * unsigned char attrib;
     */
    p = packed_header;
-   
+
    memcpy(&l, p, sizeof(l));
    header->header_len=ntohl(l);
    p+=sizeof(l);
@@ -159,7 +159,7 @@ static int unpack_header(PC3RecordHeader *header, unsigned char *packed_header)
    memcpy(&l, p, sizeof(l));
    header->header_version=ntohl(l);
    p+=sizeof(l);
-   
+
    if (header->header_version > 2) {
       jp_logf(LOG_WARN, "Unknown PC header version = %d\n", header->header_version);
    }
@@ -167,21 +167,21 @@ static int unpack_header(PC3RecordHeader *header, unsigned char *packed_header)
    memcpy(&l, p, sizeof(l));
    header->rec_len=ntohl(l);
    p+=sizeof(l);
-   
+
    memcpy(&l, p, sizeof(l));
    header->unique_id=ntohl(l);
    p+=sizeof(l);
-   
+
    memcpy(&l, p, sizeof(l));
    header->rt=ntohl(l);
    p+=sizeof(l);
-   
+
    memcpy(&(header->attrib), p, sizeof(unsigned char));
    p+=sizeof(unsigned char);
 
    return 0;
 }
-   
+
 /* FIXME: Add jp_ and document. */
 int read_header(FILE *pc_in, PC3RecordHeader *header)
 {
@@ -252,14 +252,14 @@ int jp_install_remove_line(int deleted_line)
       jp_logf(LOG_DEBUG, "failed opening install_file\n");
       return -1;
    }
-   
+
    out = jp_open_home_file("jpilot_to_install.tmp", "w");
    if (!out) {
       fclose(in);
       jp_logf(LOG_DEBUG, "failed opening install_file.tmp\n");
       return -1;
    }
-   
+
    for (line_count=0; (!feof(in)); line_count++) {
       line[0]='\0';
       Pc = fgets(line, 1000, in);
@@ -276,7 +276,7 @@ int jp_install_remove_line(int deleted_line)
    }
    fclose(in);
    fclose(out);
-   
+
    rename_file("jpilot_to_install.tmp", "jpilot_to_install");
 
    return 0;
@@ -286,7 +286,7 @@ int jp_install_append_line(char *line)
 {
    FILE *out;
    int r;
-   
+
    out = jp_open_home_file("jpilot_to_install", "a");
    if (!out) {
       return -1;
@@ -305,19 +305,17 @@ int jp_install_append_line(char *line)
 /*returns 1 if found */
 /*        0 if eof */
 static int find_next_offset(mem_rec_header *mem_rh, long fpos,
-		     unsigned int *next_offset,
-		     unsigned char *attrib, unsigned int *unique_id)
+			    long *next_offset,
+			    unsigned char *attrib, unsigned int *unique_id)
 {
    mem_rec_header *temp_mem_rh;
    unsigned char found = 0;
    unsigned long found_at;
 
-   found_at=0xFFFFFF;
+   found_at=0x1000000;
    for (temp_mem_rh=mem_rh; temp_mem_rh; temp_mem_rh = temp_mem_rh->next) {
       if ((temp_mem_rh->offset > fpos) && (temp_mem_rh->offset < found_at)) {
 	 found_at = temp_mem_rh->offset;
-	 /* *attrib = temp_mem_rh->attrib; */
-	 /* *unique_id = temp_mem_rh->unique_id; */
       }
       if ((temp_mem_rh->offset == fpos)) {
 	 found = 1;
@@ -364,14 +362,14 @@ int jp_free_DB_records(GList **br_list)
    }
    g_list_free(*br_list);
    *br_list=NULL;
-   
+
    return 0;
 }
 
 /*These next 2 functions were copied from pi-file.c in the pilot-link app */
 /* Exact value of "Jan 1, 1970 0:00:00 GMT" - "Jan 1, 1904 0:00:00 GMT" */
 #define PILOT_TIME_DELTA (unsigned)(2082844800)
- 
+
 static time_t
 pilot_time_to_unix_time (unsigned long raw_time)
 {
@@ -399,7 +397,7 @@ static unsigned int bytes_to_bin(unsigned char *bytes, unsigned int num_bytes)
 static int raw_header_to_header(RawDBHeader *rdbh, DBHeader *dbh)
 {
    unsigned long temp;
-   
+
    strncpy(dbh->db_name, rdbh->db_name, 31);
    dbh->db_name[31] = '\0';
    dbh->flags = bytes_to_bin(rdbh->flags, 2);
@@ -421,7 +419,7 @@ static int raw_header_to_header(RawDBHeader *rdbh, DBHeader *dbh)
    dbh->unique_id_seed[4] = '\0';
    dbh->next_record_list_id = bytes_to_bin(rdbh->next_record_list_id, 4);
    dbh->number_of_records = bytes_to_bin(rdbh->number_of_records, 2);
-   
+
    return 0;
 }
 
@@ -483,7 +481,7 @@ int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
       }
    }
    fclose(in);
-   
+
    *buf_size=rec_size;
 
    return 0;
@@ -501,7 +499,7 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
    if (br==NULL) {
       return -1;
    }
-   
+
    g_snprintf(PC_name, 255, "%s.pc3", DB_name);
 
    if ((br->rt==DELETED_PALM_REC) || (br->rt==MODIFIED_PALM_REC)) {
@@ -543,7 +541,7 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
       }
       fclose(pc_in);
       return -1;
-	 
+
     case PALM_REC:
       jp_logf(LOG_DEBUG, "Deleteing Palm ID %d\n", br->unique_id);
       pc_in=jp_open_home_file(PC_name, "a");
@@ -559,7 +557,7 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
       }
 
       header.rec_len = br->size;
-      
+
       jp_logf(LOG_DEBUG, "writing header to pc file\n");
       write_header(pc_in, &header);
       /*todo write the real appointment from palm db */
@@ -577,7 +575,7 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
 
    return 0;
 }
-   
+
 int jp_pc_write(char *DB_name, buf_rec *br)
 {
    PC3RecordHeader header;
@@ -589,12 +587,12 @@ int jp_pc_write(char *DB_name, buf_rec *br)
 
    g_snprintf(PC_name, 255, "%s.pc3", DB_name);
    PC_name[255]='\0';
-   
+
    get_next_unique_pc_id(&next_unique_id);
 #ifdef JPILOT_DEBUG
    jp_logf(LOG_DEBUG, "next unique id = %d\n",next_unique_id);
 #endif
-   
+
    out = jp_open_home_file(PC_name, "a");
    if (!out) {
       jp_logf(LOG_WARN, "Error opening %s\n", PC_name);
@@ -606,13 +604,13 @@ int jp_pc_write(char *DB_name, buf_rec *br)
    header.attrib=br->attrib;
    header.unique_id=next_unique_id;
    br->unique_id=next_unique_id;
-   
+
    len = pack_header(&header, packed_header);
    write_header(out, &header);
    fwrite(br->buf, header.rec_len, 1, out);
 
    fclose(out);
-   
+
    return 0;
 }
 
@@ -621,7 +619,7 @@ static int pc_read_next_rec(FILE *in, buf_rec *br)
    PC3RecordHeader header;
    int rec_len, num;
    char *record;
-   
+
    if (feof(in)) {
       return JPILOT_EOF;
    }
@@ -664,10 +662,12 @@ int jp_read_DB_files(char *DB_name, GList **records)
    FILE *pc_in;
    char *buf;
    GList *temp_list;
+   GList *end_of_list;
    int num_records, recs_returned, i, num, r;
-   unsigned int offset, prev_offset, next_offset, rec_size;
+   long offset, prev_offset, next_offset, rec_size;
    int out_of_order;
-   long fpos;  /*file position indicator */
+   long fpos, fend;  /*file position indicator */
+   int ret;
    unsigned char attrib;
    unsigned int unique_id;
    mem_rec_header *mem_rh, *temp_mem_rh, *last_mem_rh;
@@ -678,8 +678,10 @@ int jp_read_DB_files(char *DB_name, GList **records)
    char PDB_name[256];
    char PC_name[256];
 
+   jp_logf(LOG_DEBUG, "Entering jp_read_DB_files\n");
+
    mem_rh = last_mem_rh = NULL;
-   *records = NULL;
+   *records = end_of_list = NULL;
    recs_returned = 0;
 
    g_snprintf(PDB_name, 255, "%s.pdb", DB_name);
@@ -714,7 +716,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
    num_records = dbh.number_of_records;
    out_of_order = 0;
    prev_offset = 0;
-   
+
    for (i=1; i<num_records+1; i++) {
       num = fread(&rh, sizeof(record_header), 1, in);
       if (num != 1) {
@@ -760,10 +762,9 @@ int jp_read_DB_files(char *DB_name, GList **records)
    }
 
    temp_mem_rh = mem_rh;
-
    if (num_records) {
       if (out_of_order) {
-	 find_next_offset(mem_rh, 0, &next_offset, &attrib, &unique_id);
+	 ret=find_next_offset(mem_rh, 0, &next_offset, &attrib, &unique_id);
       } else {
 	 if (mem_rh) {
 	    next_offset = mem_rh->offset;
@@ -775,15 +776,27 @@ int jp_read_DB_files(char *DB_name, GList **records)
       while(!feof(in)) {
 	 fpos = ftell(in);
 	 if (out_of_order) {
-	    find_next_offset(mem_rh, fpos, &next_offset, &attrib, &unique_id);
+	    ret = find_next_offset(mem_rh, fpos, &next_offset, &attrib, &unique_id);
+	    if (!ret) {
+	       /* Next offset should be end of file */
+	       fseek(in, 0, SEEK_END);
+	       fend = ftell(in);
+	       fseek(in, fpos, SEEK_SET);
+	       next_offset = fend + 1;
+	    }
 	 } else {
-	    next_offset = 0xFFFFFF;
 	    if (temp_mem_rh) {
 	       attrib = temp_mem_rh->attrib;
 	       unique_id = temp_mem_rh->unique_id;
 	       if (temp_mem_rh->next) {
 		  temp_mem_rh = temp_mem_rh->next;
 		  next_offset = temp_mem_rh->offset;
+	       } else {
+		  /* Next offset should be end of file */
+		  fseek(in, 0, SEEK_END);
+		  fend = ftell(in);
+		  fseek(in, fpos, SEEK_SET);
+		  next_offset = fend + 1;
 	       }
 	    }
 	 }
@@ -795,8 +808,12 @@ int jp_read_DB_files(char *DB_name, GList **records)
 #endif
 	 buf = malloc(rec_size);
 	 if (!buf) break;
-	 num = fread(buf, rec_size, 1, in);
-	 if ((num != 1)) {
+	 num = fread(buf, 1, rec_size, in);
+	 if (num<rec_size) {
+	    rec_size=num;
+	    buf = realloc(buf, rec_size);
+	 }
+	 if ((num < 1)) {
 	    if (ferror(in)) {
 	       jp_logf(LOG_WARN, "Error reading %s 5\n", PDB_name);
 	       free(buf);
@@ -815,12 +832,24 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	 temp_br->buf = buf;
 	 temp_br->size = rec_size;
 
-	 *records = g_list_append(*records, temp_br);
-	 
+	 /* g_list_append parses the list to get to the end on every call.
+	  * To speed it up we have to give the last record
+	  */
+	 if (*records==NULL) {
+	    *records = g_list_append(*records, temp_br);
+	    end_of_list=*records;
+	 } else {
+	    *records = g_list_append(end_of_list, temp_br);
+	    if (end_of_list->next) {
+	       end_of_list=end_of_list->next;
+	    }
+	 }
+
 	 recs_returned++;
       }
    }
    fclose(in);
+
    free_mem_rec_header(&mem_rh);
    /* */
    /* Get the appointments out of the PC database */
@@ -847,18 +876,29 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	  &&(temp_br->rt!=MODIFIED_PALM_REC)
 	  &&(temp_br->rt!=DELETED_DELETED_PALM_REC)) {
 
-	 *records = g_list_append(*records, temp_br);
-	 
+	 /* g_list_append parses the list to get to the end on every call.
+	  * To speed it up we have to give the last record
+	  */
+	 if (*records==NULL) {
+	    *records = g_list_append(*records, temp_br);
+	    end_of_list=*records;
+	 } else {
+	    *records = g_list_append(end_of_list, temp_br);
+	    if (end_of_list->next) {
+	       end_of_list=end_of_list->next;
+	    }
+	 }
+
 	 recs_returned++;
       }
       if ((temp_br->rt==DELETED_PALM_REC) || (temp_br->rt==MODIFIED_PALM_REC)) {
 	 temp_list=*records;
 	 if (*records) {
-	    while(temp_list->next) {
-	       temp_list=temp_list->next;
+	    while(temp_list->prev) {
+	       temp_list=temp_list->prev;
 	    }
 	 }
-	 for (; temp_list; temp_list=temp_list->prev) {
+	 for (; temp_list; temp_list=temp_list->next) {
 	    if (((buf_rec *)temp_list->data)->unique_id == temp_br->unique_id) {
 	       ((buf_rec *)temp_list->data)->rt = temp_br->rt;
 	    }
@@ -867,7 +907,8 @@ int jp_read_DB_files(char *DB_name, GList **records)
    }
    fclose(pc_in);
 
-   jp_logf(LOG_DEBUG, "Leaving get_recs\n");
+   jp_logf(LOG_DEBUG, "Leaving jp_read_DB_files\n");
 
    return recs_returned;
 }
+
