@@ -1,6 +1,7 @@
 /* monthview_gui.c
+ * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include "utils.h"
+#include "print.h"
 #include "prefs.h"
 #include "log.h"
 #include "datebook.h"
@@ -54,8 +56,7 @@ static gboolean cb_destroy(GtkWidget *widget)
 }
 
 static void
-cb_quit(GtkWidget *widget,
-	gpointer   data)
+cb_quit(GtkWidget *widget, gpointer data)
 {
    window = NULL;
    gtk_widget_destroy(data);
@@ -79,6 +80,27 @@ cb_month_move(GtkWidget *widget,
    display_months_appts(&glob_month_date, glob_month_texts);
 }
 
+/*----------------------------------------------------------------------
+ * cb_month_print	This is where month printing is kicked off from
+ *----------------------------------------------------------------------*/
+
+static void cb_month_print(GtkWidget *widget, gpointer data)
+{
+   long paper_size;
+
+   jpilot_logf(LOG_DEBUG, "cb_month_print called\n");
+   if (print_gui(window, DATEBOOK, 3, 0x04) == DIALOG_SAID_PRINT) {
+      get_pref(PREF_PAPER_SIZE, &paper_size, NULL);
+      if (paper_size==1) {
+	 print_months_appts(&glob_month_date, PAPER_A4);
+      } else {
+	 print_months_appts(&glob_month_date, PAPER_Letter);
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+
 static void
 cb_enter_notify(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
@@ -99,9 +121,9 @@ cb_enter_notify(GtkWidget *widget, GdkEvent *event, gpointer data)
       return;
    }
    prev_day = GPOINTER_TO_INT(data)+1;
-   
+
    a_list = NULL;
-   
+
    memcpy(&date, &glob_month_date, sizeof(struct tm));
    date.tm_mday=GPOINTER_TO_INT(data)+1;
    mktime(&date);
@@ -148,7 +170,7 @@ cb_enter_notify(GtkWidget *widget, GdkEvent *event, gpointer data)
 void create_month_boxes()
 {
    int i;
-   
+
    for (i=0; i<6; i++) {
       glob_hbox_month_row[i] = gtk_hbox_new(TRUE, 0);
       gtk_box_pack_start(GTK_BOX(glob_month_vbox), glob_hbox_month_row[i], TRUE, TRUE, 0);
@@ -158,7 +180,7 @@ void create_month_boxes()
 void destroy_month_boxes()
 {
    int i;
-   
+
    for (i=0; i<6; i++) {
       if (glob_hbox_month_row[i]) {
 	 gtk_widget_destroy(glob_hbox_month_row[i]);
@@ -182,7 +204,7 @@ void create_month_texts()
    /* Month name label */
    strftime(str, 60, "%B %Y", &glob_month_date);
    gtk_label_set_text(GTK_LABEL(glob_month_month_label), str);
-   
+
    get_month_info(glob_month_date.tm_mon, 1, glob_month_date.tm_year, &dow, &ndim);
 
    get_pref(PREF_FDOW, &fdow, &str_fdow);
@@ -196,7 +218,7 @@ void create_month_texts()
       gtk_box_pack_start(GTK_BOX(glob_hbox_month_row[0]), hbox, TRUE, TRUE, 0);
       column++;
    }
-     
+
    for (n=0; n<ndim; n++) {
       glob_month_texts[n] = gtk_text_new(NULL, NULL);
       gtk_widget_set_usize(GTK_WIDGET(glob_month_texts[n]), 110, 90);
@@ -241,6 +263,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
    int dow;
    int ndim;
    int n;
+   int mask;
 #ifdef USE_DB3
    int ret;
    int category;
@@ -250,6 +273,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
 
    a_list = NULL;
    text = day_texts;
+   mask=0;
 /*
    get_pref(PREF_CHAR_SET, &char_set, NULL);
    if (char_set==CHAR_SET_1250) {
@@ -265,7 +289,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
 
    get_month_info(date.tm_mon, 1, date.tm_year, &dow, &ndim);
 
-   weed_datebook_list(&a_list, date.tm_mon, date.tm_year);
+   weed_datebook_list(&a_list, date.tm_mon, date.tm_year, &mask);
 
    for (n=0, date.tm_mday=1; date.tm_mday<=ndim; date.tm_mday++, n++) {
       date.tm_sec=0;
@@ -311,7 +335,7 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
       }
    }
    free_AppointmentList(&a_list);
-   
+
    gtk_widget_show_all(window);
    return 0;
 }
@@ -344,13 +368,13 @@ void monthview_gui(struct tm *date_in)
    if (window) {
       return;
    }
-   
+
    for (i=0; i<6; i++) {
       glob_hbox_month_row[i]=NULL;
    }
 
    memcpy(&glob_month_date, date_in, sizeof(struct tm));
-   
+
    get_pref(PREF_FDOW, &fdow, &str_fdow);
 
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -382,13 +406,19 @@ void monthview_gui(struct tm *date_in)
 		      GINT_TO_POINTER(-1));
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 3);
 
-   
+
    /* Create a "Quit" button */
    button = gtk_button_new_with_label(_("Close"));
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_quit), window);
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
 
+   /* Create a "Print" button */
+   button = gtk_button_new_with_label(_("Print"));
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_month_print), window);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
+
    /*Make a right arrow for going forward a week */
    button = gtk_button_new();
    arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
@@ -403,7 +433,7 @@ void monthview_gui(struct tm *date_in)
    strftime(str, 60, "%B %Y", &glob_month_date);
    glob_month_month_label = gtk_label_new(str);
    gtk_box_pack_start(GTK_BOX(vbox), glob_month_month_label, TRUE, TRUE, 0);
-   
+
    /* Days of the week */
    hbox = gtk_hbox_new(TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
@@ -417,7 +447,7 @@ void monthview_gui(struct tm *date_in)
    gtk_box_pack_start(GTK_BOX(vbox), glob_month_vbox, TRUE, TRUE, 0);
 
    create_month_boxes();
-     
+
    create_month_texts();
 
    big_text = gtk_text_new(NULL, NULL);
@@ -425,7 +455,6 @@ void monthview_gui(struct tm *date_in)
    gtk_box_pack_start(GTK_BOX(vbox), big_text, TRUE, TRUE, 0);
 
    display_months_appts(&glob_month_date, glob_month_texts);
- 
+
    gtk_widget_show_all(window);
 }
-
