@@ -1,6 +1,7 @@
 /* prefs_gui.c
+ * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +37,9 @@ static GtkWidget *alarm_command_entry;
 extern int glob_app;
 
 #ifdef COLORS
-/* This doesn't work quite right.  I don't know why. */
+
+/* This doesn't work quite right.  There is supposedly no way to do it in GTK. */
+
 void set_colors()
 {
    read_gtkrc_file();
@@ -47,12 +50,13 @@ void set_colors()
    gtk_widget_queue_draw(main_window);
 }
 #endif
+
 static void cb_pref_menu(GtkWidget *widget,
 			 gpointer   data)
 {
    int pref;
    int value;
-   
+
    if (!widget)
      return;
    if (!(GTK_CHECK_MENU_ITEM(widget))->active) {
@@ -62,7 +66,7 @@ static void cb_pref_menu(GtkWidget *widget,
    pref = GPOINTER_TO_INT(data);
    value = pref & 0xFF;
    pref = pref >> 8;
-   set_pref(pref, value);
+   set_pref(pref, value, NULL);
    jpilot_logf(LOG_DEBUG, "pref %d, value %d\n", pref, value);
 #ifdef COLORS
    if (pref==PREF_RCFILE) {
@@ -71,9 +75,9 @@ static void cb_pref_menu(GtkWidget *widget,
 #endif
    return;
 }
-   
 
-static int make_pref_menu(GtkWidget **pref_menu, int pref_num)
+
+int make_pref_menu(GtkWidget **pref_menu, int pref_num)
 {
    GtkWidget *menu_item;
    GtkWidget *menu;
@@ -85,17 +89,17 @@ static int make_pref_menu(GtkWidget **pref_menu, int pref_num)
    char human_text[MAX_PREF_VALUE];
    time_t ltime;
    struct tm *now;
-      
+
    time(&ltime);
    now = localtime(&ltime);
 
    *pref_menu = gtk_option_menu_new();
-   
+
    menu = gtk_menu_new();
    group = NULL;
-   
+
    get_pref(pref_num, &ivalue, &svalue);
-	    
+
    for (i=0; i<1000; i++) {
       r = get_pref_possibility(pref_num, i, format_text);
       if (r) {
@@ -113,61 +117,67 @@ static int make_pref_menu(GtkWidget **pref_menu, int pref_num)
       }
       menu_item = gtk_radio_menu_item_new_with_label(
 		     group, human_text);
-      gtk_signal_connect(GTK_OBJECT(menu_item), "activate", cb_pref_menu,
-			 GINT_TO_POINTER(((pref_num*0x100) + (i & 0xFF))));
       group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menu_item));
       gtk_menu_append(GTK_MENU(menu), menu_item);
-      
+
       if (ivalue == i) {
 	 gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), ivalue);
       }
 
+      gtk_signal_connect(GTK_OBJECT(menu_item), "activate", cb_pref_menu,
+			 GINT_TO_POINTER(((pref_num*0x100) + (i & 0xFF))));
+
       gtk_widget_show(menu_item);
    }
    gtk_option_menu_set_menu(GTK_OPTION_MENU(*pref_menu), menu);
-   
+
    return 0;
 }
 
 void cb_show_deleted(GtkWidget *widget,
 		     gpointer data)
 {
-   set_pref(PREF_SHOW_DELETED, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_SHOW_DELETED, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_show_modified(GtkWidget *widget,
 		      gpointer data)
 {
-   set_pref(PREF_SHOW_MODIFIED, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_SHOW_MODIFIED, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_highlight(GtkWidget *widget,
 		  gpointer data)
 {
-   set_pref(PREF_HIGHLIGHT, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_HIGHLIGHT, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_use_db3(GtkWidget *widget,
 		gpointer data)
 {
-   set_pref(PREF_USE_DB3, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_USE_DB3, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_sync_datebook(GtkWidget *widget,
 		      gpointer data)
 {
-   set_pref(PREF_SYNC_DATEBOOK, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_SYNC_DATEBOOK, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_sync_address(GtkWidget *widget,
 		     gpointer data)
 {
-   set_pref(PREF_SYNC_ADDRESS, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_SYNC_ADDRESS, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_sync_todo(GtkWidget *widget,
 		  gpointer data)
 {
-   set_pref(PREF_SYNC_TODO, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_SYNC_TODO, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_sync_memo(GtkWidget *widget,
 		  gpointer data)
 {
-   set_pref(PREF_SYNC_MEMO, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_SYNC_MEMO, GTK_TOGGLE_BUTTON(widget)->active, NULL);
+}
+void cb_sync_memo32(GtkWidget *widget,
+		    gpointer data)
+{
+   set_pref(PREF_SYNC_MEMO32, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 #ifdef ENABLE_PLUGINS
 void cb_sync_plugin(GtkWidget *widget,
@@ -176,13 +186,13 @@ void cb_sync_plugin(GtkWidget *widget,
    GList *plugin_list, *temp_list;
    struct plugin_s *Pplugin;
    int number;
-   
+
    number = GPOINTER_TO_INT(data);
-   
+
    plugin_list=NULL;
-   
+
    plugin_list = get_plugin_list();
-   
+
    for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
       Pplugin = (struct plugin_s *)temp_list->data;
       if (Pplugin) {
@@ -201,12 +211,12 @@ void cb_sync_plugin(GtkWidget *widget,
 void cb_open_alarm(GtkWidget *widget,
 		   gpointer data)
 {
-   set_pref(PREF_OPEN_ALARM_WINDOWS, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_OPEN_ALARM_WINDOWS, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 void cb_do_command(GtkWidget *widget,
 		   gpointer data)
 {
-   set_pref(PREF_DO_ALARM_COMMAND, GTK_TOGGLE_BUTTON(widget)->active);
+   set_pref(PREF_DO_ALARM_COMMAND, GTK_TOGGLE_BUTTON(widget)->active, NULL);
 }
 
 
@@ -216,17 +226,17 @@ static gboolean cb_destroy(GtkWidget *widget)
    char *entry_text;
    char *backups_text;
    int num_backups;
-   
+
    jpilot_logf(LOG_DEBUG, "Cleanup\n");
 
    entry_text = gtk_entry_get_text(GTK_ENTRY(port_entry));
    jpilot_logf(LOG_DEBUG, "port_entry = [%s]\n", entry_text);
-   set_pref_char(PREF_PORT, entry_text);
-   
+   set_pref(PREF_PORT, 0, entry_text);
+
    entry_text = gtk_entry_get_text(GTK_ENTRY(alarm_command_entry));
    jpilot_logf(LOG_DEBUG, "alarm_command_entry = [%s]\n", entry_text);
-   set_pref_char(PREF_ALARM_COMMAND, entry_text);
-   
+   set_pref(PREF_ALARM_COMMAND, 0, entry_text);
+
    backups_text = gtk_entry_get_text(GTK_ENTRY(backups_entry));
    jpilot_logf(LOG_DEBUG, "backups_entry = [%s]\n", backups_text);
    num_backups = atoi(backups_text);
@@ -236,9 +246,9 @@ static gboolean cb_destroy(GtkWidget *widget)
    if (num_backups > 99) {
       num_backups = 99;
    }
-   set_pref(PREF_NUM_BACKUPS, num_backups);
-   
-   write_rc_file();
+   set_pref(PREF_NUM_BACKUPS, num_backups, NULL);
+
+   pref_write_rc_file();
 
    window = NULL;
    if (glob_app==DATEBOOK) {
@@ -280,7 +290,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    struct plugin_s *Pplugin;
    extern unsigned char skip_plugins;
 #endif
-   
+
    jpilot_logf(LOG_DEBUG, "cb_prefs_gui\n");
    if (window) {
       jpilot_logf(LOG_DEBUG, "pref_window is already up\n");
@@ -415,7 +425,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    make_pref_menu(&pref_menu, PREF_RCFILE);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
 			     1, 2, 0, 1);
-   
+
    get_pref(PREF_RCFILE, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
@@ -565,9 +575,21 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_signal_connect(GTK_OBJECT(checkbutton), 
 		      "clicked", GTK_SIGNAL_FUNC(cb_sync_memo), NULL);
 
+   /*Show sync Memo32 check box */
+   checkbutton = gtk_check_button_new_with_label
+     (_("Sync memo32 (pedit32)"));
+   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
+   get_pref(PREF_SYNC_MEMO32, &ivalue, &cstr);
+   gtk_widget_show(checkbutton);
+   if (ivalue) {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+   }
+   gtk_signal_connect(GTK_OBJECT(checkbutton), 
+		      "clicked", GTK_SIGNAL_FUNC(cb_sync_memo32), NULL);
+
 #ifdef  ENABLE_PLUGINS
    if (!skip_plugins) {
-     
+
       plugin_list=NULL;
 
       plugin_list = get_plugin_list();
