@@ -1,6 +1,7 @@
 /* print.c
+ * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 2000 by Judd Montgomery
+ * Copyright (C) 2000-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,19 +21,36 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+#include <ctype.h>
+#include <stdlib.h>
 #include "datebook.h"
 #include "address.h"
 #include "todo.h"
 #include "sync.h"
 #include "prefs.h"
 #include "print.h"
+#include "print_logo.h"
+#include "print_headers.h"
 #include "log.h"
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
 
 static FILE *out;
+static int first_hour, first_min, last_hour, last_min;
+extern int datebook_category;
 int fill_in(struct tm *date, AppointmentList *a_list);
+void ps_strncat(char *dest, char *src, int n);
+
+char *PaperSizes[] = { "Letter", "Legal", "Statement", "Tabloid", "Ledger",
+                       "Folio", "Quarto", "7x9", "9x11", "9x12", "10x13",
+                       "10x14", "Executive", "A0", "A1", "A2", "A3", "A4",  
+                       "A5", "A6", "A7", "A8", "A9", "A10", "B0", "B1", "B2",
+                       "B3", "B4", "B5", "B6", "B7", "B8", "B9", "B10",
+                       "ISOB0", "ISOB1", "ISOB2", "ISOB3", "ISOB4", "ISOB5",
+                       "ISOB6", "ISOB7", "ISOB8", "ISOB9", "ISOB10", "C0",
+                       "C1", "C2", "C3", "C4", "C5", "C6", "C7", "DL",
+                       "Filo" };
 
 #ifdef JPILOT_PRINTABLE
 #define FLAG_CHAR 'A'
@@ -46,7 +64,7 @@ FILE *print_open()
 {
    long ivalue;
    const char *command;
-   
+
    get_pref(PREF_PRINT_COMMAND, &ivalue, &command);
    if (command) {
       return popen(command, "w");
@@ -74,95 +92,34 @@ int courier_bold_12()
    return 0;
 }
 
-int gsave()
-{
-   fprintf(out, "gsave\n");
-   return 0;
-}
-
-int grestore()
-{
-   fprintf(out, "grestore\n");
-   return 0;
-}
-
-int rotate(float deg)
-{
-   fprintf(out, "%f rotate\n", deg);
-   return 0;
-}
-
-int box(float x1, float y1, float x2, float y2)
-{
-   fprintf(out, "newpath\n");
-   fprintf(out, "%g inch %g inch moveto\n", x1, y1);
-   fprintf(out, "%g inch %g inch lineto\n", x1, y2);
-   fprintf(out, "%g inch %g inch lineto\n", x2, y2);
-   fprintf(out, "%g inch %g inch lineto\n", x2, y1);
-   fprintf(out, "closepath\n");
-   return 0;
-}
-
 int clip_to_box(float x1, float y1, float x2, float y2)
 {
-   box(x1, y1, x2, y2);
-   fprintf(out, "clip\n");
+   fprintf(out, "%g inch %g inch %g inch %g inch rectclip\n",
+           x1, y1, x2 - x1, y2 - y1);
    return 0;
 }
 
 int puttext(float x, float y, char *text)
 {
-   fprintf(out, "%g inch %g inch moveto\n", x, y);
-   fprintf(out, "(%s) show\n", text);
-   return 0;
-}
+   int len;
+   char *buf;
 
-int setfont(char *name, int points)
-{
-   fprintf(out, "/%s findfont\n", name);
-   fprintf(out, "%d scalefont\n", points);
-   fprintf(out, "setfont\n");
-   return 0;
-}
-
-int linefromto(float x1, float y1, float x2, float y2)
-{
-   fprintf(out, "%g inch %g inch moveto\n", x1, y1);
-   fprintf(out, "%g inch %g inch lineto\n", x2, y2);
-   return 0;
-}
-
-int setgray(float n)
-{
-   fprintf(out, "%g setgray\n", n);
-   return 0;
-}
-
-int fill()
-{
-   fprintf(out, "fill\n");
-   return 0;
-}
-
-int stroke()
-{
-   fprintf(out, "stroke\n");
-   return 0;
-}
-
-int showpage()
-{
-   fprintf(out, "showpage\n");
+   len = strlen(text);
+   buf = (char *) malloc(2 * len + 1);
+   bzero(buf, 2 * len + 1);
+   ps_strncat(buf, text, 2 * len);
+   fprintf(out, "%g inch %g inch moveto (%s) show\n", x, y, buf);
+   free(buf);
    return 0;
 }
 
 int header()
 {
    time_t ltime;
-   
+
    time(&ltime);
    fprintf(out,
-	   "%%!PS-Adobe-2.0 EPSF-2.0\n"
+	   "%%!PS-Adobe-2.0\n"
 	   "%%%%Creator: J-Pilot\n"
 	   "%%%%CreationDate: %s\n"
            "%%%%DocumentData: Clean7Bit\n"
@@ -195,48 +152,22 @@ int header()
            "%%%%EndProlog\n"
            "%%%%Page: 1 1\n\n",
 	   ctime(&ltime));
-   return 0;
-}
 
-int newpath()
-{
-   fprintf(out, "newpath\n");
-   return 0;
-}
-
-int closepath()
-{
-   fprintf(out, "closepath\n");
-   return 0;
-}
-
-int setlinewidth(int w)
-{
-   fprintf(out, "%d setlinewidth\n", w);
-   return 0;
-}
-
-int moveto(float x, float y)
-{
-   fprintf(out, "%g inch %g inch moveto\n", x, y);
-   return 0;
-}
-
-int lineto(float x, float y)
-{
-   fprintf(out, "%g inch %g inch lineto\n", x, y);
+   fprintf(out, "/PageSize (%s) def\n\n", PaperSizes[PAPER_Letter]);
+   print_common_header(out);
+   fprintf(out, "595 612 div 842 792 div Min dup scale %% HACK!!!! (CMB)\n");
+           /* This hack pre-scales to compensate for the standard scaling
+              mechanism below, to avoid me having to redo the layout of
+              the dayview for the A4 standard size page. */
    return 0;
 }
 
 int print_dayview(struct tm *date, AppointmentList *a_list)
 {
-   float x, y;
-   int hour;
    char str[80];
    char datef[80];
    time_t ltime;
    struct tm *now;
-   struct tm hours;
    const char *svalue;
    long ivalue;
 
@@ -246,94 +177,39 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
 
    header();
    /* Draw the 2 gray columns and header block */
-   newpath();
-   setlinewidth(0);
-   moveto(0.5, 3.5);
-   lineto(0.5, 10.5);
-   lineto(8, 10.5);
-   lineto(8, 9.5);
-   lineto(5, 9.5);
-   lineto(5, 3.5);
-   lineto(4.25, 3.5);
-   lineto(4.25, 9.5);
-   lineto(1.25, 9.5);
-   lineto(1.25, 3.5);
-   closepath();
-   setgray(0.85);
-   fill();
-   stroke();
-
-   /* make a couple of lines in the header box */
-   newpath();
-   setlinewidth(1);
-   setgray(0);
-   linefromto(0.5, 10.5, 8, 10.5);
-   stroke();
-   newpath();
-   setlinewidth(2);
-   linefromto(0.5, 9.5, 8, 9.5);
-   stroke();
-   newpath();
-   setlinewidth(1);
-   linefromto(0.5, 9.5625, 8, 9.5625);
-   stroke();
-
-   setfont("Times-Bold-ISOLatin1", 20);
+   print_day_header(out);
 
    /* Put the Month name up */
-   newpath();
-   setgray(0);
+   fprintf(out, "/Times-Bold-ISOLatin1 findfont 20 scalefont setfont\n"
+           "newpath 0 setgray\n");
    get_pref(PREF_LONGDATE, &ivalue, &svalue);
    /* strftime(str, 80, "%B %d, %Y", date); */
    strftime(str, 80, svalue, date);
    puttext(0.5, 10.25, str);
 
    /* Put the weekday name up */
-   setfont("Times-Roman-ISOLatin1", 15);
-   strftime(str, 80, "%A", date);   
+   fprintf(out, "/Times-Roman-ISOLatin1 findfont 15 scalefont setfont\n");
+   strftime(str, 80, "%A", date);
    puttext(0.5, 10, str);
 
-   setfont("Times-Roman-ISOLatin1", 14);
-   hour=0;
-   x=0.5; y=9.15;
-   bzero(&hours, sizeof(hours));
-   get_pref_time_no_secs(datef);
-   for (hour=0; hour<24; hour++) {
-      if (hour==12) {
-	 x=4.25;
-	 y=9.15;
-      }
-      hours.tm_hour=hour;
-      strftime(str, 80, datef, &hours);
-      puttext(x, y, str);
-      y = y-0.5;
-   }
-   stroke();
-
-   /* draw horizontal lines across hours */
-   newpath();
-   setlinewidth(0);
-   for (y=9.0; y>3.25; y=y-0.5) {
-      linefromto(1.25, y, 4.25, y);
-      linefromto(5.0, y, 8.0, y);
-   }
-   stroke();
-   
-   newpath();
-   setfont("Times-Roman-ISOLatin1", 10);
+   /* Put the time of printing up */
+   fprintf(out, "newpath\n"
+           "/Times-Roman-ISOLatin1 findfont 10 scalefont setfont\n");
 
    time(&ltime);
    now = localtime(&ltime);
    get_pref(PREF_SHORTDATE, &ivalue, &svalue);
-   strftime(str, 80, svalue, now);
+   g_snprintf(datef, 80, "%s %s", "Printed on: ", svalue);
+   strftime(str, 80, datef, now);
    puttext(0.5, 0.9, str);
    puttext(7.5, 0.9, "J-Pilot");
-   stroke();
+   fprintf(out, "stroke\n");
+   print_logo(out, 40, 90, 0.35);
 
    fill_in(date, a_list);
-   
-   showpage();
-   
+
+   fprintf(out, "showpage\n");
+
    fprintf(out,
 	   "%%%%PageTrailer\n"
 	   "%%%%Trailer\n"
@@ -346,127 +222,6 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
    return 0;
 }
 
-int print_monthview(struct tm *date)
-{
-   float x, y;
-   int hour;
-   char str[80];
-   time_t ltime;
-   struct tm *now;
-   const char *svalue;
-   long ivalue;
-
-   out = fopen("ps_out.ps", "w");
-
-   header();
-   /* Draw the 2 gray columns and header block */
-   rotate(90);
-   newpath();
-   setlinewidth(0);
-   moveto(0.5, 3.5);
-   lineto(0.5, 10.5);
-   lineto(8, 10.5);
-   lineto(8, 9.5);
-   lineto(5, 9.5);
-   lineto(5, 3.5);
-   lineto(4.25, 3.5);
-   lineto(4.25, 9.5);
-   lineto(1.25, 9.5);
-   lineto(1.25, 3.5);
-   closepath();
-   setgray(0.85);
-   fill();
-   stroke();
-
-   /* make a couple of lines in the header box */
-   newpath();
-   setlinewidth(1);
-   setgray(0);
-   linefromto(0.5, 10.5, 8, 10.5);
-   stroke();
-   newpath();
-   setlinewidth(2);
-   linefromto(0.5, 9.5, 8, 9.5);
-   stroke();
-   newpath();
-   setlinewidth(1);
-   linefromto(0.5, 9.5625, 8, 9.5625);
-   stroke();
-
-   setfont("Times-Bold-ISOLatin1", 20);
-
-   /* Put the Month name up */
-   newpath();
-   setgray(0);
-   strftime(str, 80, "%B %d, %Y", date);
-   puttext(0.5, 10.25, str);
-
-   /* Put the weekday name up */
-   setfont("Times-Roman-ISOLatin1", 15);
-   strftime(str, 80, "%A", date);   
-   puttext(0.5, 10, str);
-
-   hour=0;
-   x=0.5; y=9.15;
-   for (hour=0; hour<24; hour++) {
-      if (hour==12) {
-	 x=4.25;
-	 y=9.15;
-      }
-      if (hour<12) {
-	 sprintf(str, "%d:00am", hour);
-      } else {
-	 sprintf(str, "%d:00pm", hour-12);
-      }
-      puttext(x, y, str);
-      y = y-0.5;
-   }
-   stroke();
-
-   /* draw horizontal lines across hours */
-   newpath();
-   setlinewidth(0);
-   for (y=9.0; y>3.25; y=y-0.5) {
-      linefromto(1.25, y, 4.25, y);
-      linefromto(5.0, y, 8.0, y);
-   }
-   stroke();
-   
-   newpath();
-   setfont("Times-Roman-ISOLatin1", 10);
-
-   time(&ltime);
-   now = localtime(&ltime);
-   get_pref(PREF_SHORTDATE, &ivalue, &svalue);
-   strftime(str, 80, svalue, now);
-   puttext(0.5, 0.9, str);
-   puttext(7.5, 0.9, "J-Pilot");
-   stroke();
-/*
-   gsave();
-   clip_to_box(1.25, 0.5, 4.25, 9.5);
-   
-   setfont("Times-Roman", 10);
-   puttext(1.25, 5.4, "8:00 Arrive at work");
-   puttext(1.25, 5.3, "8:00 Meeting");
-   puttext(1.25, 5.2, "8:15 Coffee Break!");
-   puttext(1.25, 5.1, "8:30 This is a very long appontment which should be too long");
-   puttext(1.25, 5.0, "8:59 Last appointment ***");
-   grestore();
-
-   gsave();
-   clip_to_box(5.0, 0.5, 8.0, 9.5);
-   puttext(5.0, 5.1, "8:30 This is a very long appontment which should be too long");
-   grestore();
-*/ 
-   
-   showpage();
-   
-   fclose(out);
-   
-   return 0;
-}
-
 int fill_in(struct tm *date, AppointmentList *a_list)
 {
    AppointmentList *temp_al;
@@ -488,12 +243,12 @@ int fill_in(struct tm *date, AppointmentList *a_list)
    for (i=0; i<24; i++) {
       hours[i]=0;
    }
-   
+
    /* We have to go through them twice, once for AM, and once for PM
     * This is because of the clipping */
    for (i=0; i<2; i++) {
       am=i%2;
-      gsave();
+      fprintf(out, "gsave\n");
       if (am) {
 	 clip_to_box(1.25, 0.5, 4.25, 9.5);
       } else {
@@ -552,24 +307,31 @@ int fill_in(struct tm *date, AppointmentList *a_list)
 	    jpilot_logf(LOG_WARN, "Too many appointments, dropping one\n");
 	 }
       }
-      grestore();
+      fprintf(out, "grestore\n");
    }
-   free_AppointmentList(&a_list);
 
    return 0;
 }
 
-int print_datebook(struct tm *date, AppointmentList *a_list)
+int print_days_appts(struct tm *date)
 {   
+   AppointmentList *a_list;
+
    out = print_open();
    if (!out) {
       return -1;
    }
 
+   a_list = NULL;
+
+   get_days_appointments2(&a_list, date, 2, 2, 2);
+
    print_dayview(date, a_list);
 
    print_close(out);
-   
+
+   free_AppointmentList(&a_list);
+
    return 0;
 }
 
@@ -600,11 +362,377 @@ int f_indent_print(FILE *f, int indent, char *str) {
    return 0;
 }
 
-/*
+/*----------------------------------------------------------------------
+ * ps_strncat	Escapes brackets for printing in PostScript strings
+ *----------------------------------------------------------------------*/
+
+void ps_strncat(char *dest, char *src, int n)
+{
+   int i = 0, j = 0;
+   char *dest2;
+   dest2 = strchr(dest, '\0');
+   while (j < n) {
+      if (src[i] == '\0') {
+	 dest2[j]='\0';
+	 break;
+      }
+      if (strchr("()", src[i]) != NULL) {
+	 dest2[j] = '\\';
+	 j++;
+      }
+      dest2[j] = src[i];
+      i++;
+      j++;
+   }
+}
+
+/*----------------------------------------------------------------------
+ * days_in_mon	Returns the number of days in the month containing the
+ *		date passed in.
+ *----------------------------------------------------------------------*/
+
+int days_in_mon(struct tm *date)
+{
+    int days_in_month[]={ 31,28,31,30,31,30,31,31,30,31,30,31 };
+
+    if ((date->tm_year%4 == 0) &&
+        !(((date->tm_year+1900)%100==0) && ((date->tm_year+1900)%400!=0))) {
+        days_in_month[1]++;
+    }
+    return(days_in_month[date->tm_mon]);
+}
+
+/*----------------------------------------------------------------------
+ * print_months_appts	Function to print the current month's
+ *			appointments.
+ *----------------------------------------------------------------------*/
+
+char *MonthNames[] = { 
+   "January", "February", "March", "April", "May", "June", "July",
+   "August", "September", "October", "November", "December"
+};
+
+int print_months_appts(struct tm *date_in, PaperSize paper_size)
+{
+   AppointmentList *a_list;
+   AppointmentList *temp_al;
+   struct tm date;
+   char desc[100];
+   time_t ltime;
+   int dow;
+   int ndim;
+   int n;
+   const char *svalue;
+   long fdow;
+   int mask;
+#ifdef USE_DB3
+   int ret;
+   int category;
+   int cat_bit;
+   long use_db3_tags;
+   get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+#endif
+   /*------------------------------------------------------------------
+    * Set up the PostScript output file, and print the header to it.
+    *------------------------------------------------------------------*/
+   mask=0;
+
+   time(&ltime);
+
+   if (! (out = print_open())) return(-1);
+   fprintf(out, "%%!PS-Adobe-2.0\n\n"
+	   "/PageSize (%s) def\n\n", PaperSizes[paper_size]);
+   print_month_header(out);
+
+   /*------------------------------------------------------------------
+    * Extract the appointments
+    *------------------------------------------------------------------*/
+   a_list = NULL;
+   memcpy(&date, date_in, sizeof(struct tm));
+   /* Get all of the appointments */
+
+   get_days_appointments2(&a_list, NULL, 2, 2, 2);
+   get_month_info(date.tm_mon, 1, date.tm_year, &dow, &ndim);
+   weed_datebook_list(&a_list, date.tm_mon, date.tm_year, &mask);
+
+   /*------------------------------------------------------------------
+    * Loop through the days in the month, printing appointments
+    *------------------------------------------------------------------*/
+   date.tm_mday=1;
+   date.tm_sec=0;
+   date.tm_min=0;
+   date.tm_hour=11;
+   date.tm_isdst=-1;
+   mktime(&date);
+
+   get_pref(PREF_FDOW, &fdow, &svalue);
+
+   fprintf(out,
+	   "(%s, %d) %d (%s) (%s version %s) %ld InitialisePage\n\n",
+	   MonthNames[date_in->tm_mon], date_in->tm_year + 1900,
+	   date.tm_wday,
+	   ctime(&ltime),
+	   PN, VERSION, fdow);
+
+   for (n=0, date.tm_mday=1; date.tm_mday<=ndim; date.tm_mday++, n++) {
+      date.tm_sec=0;
+      date.tm_min=0;
+      date.tm_hour=11;
+      date.tm_isdst=-1;
+      date.tm_wday=0;
+      date.tm_yday=1;
+      mktime(&date);
+
+      fprintf(out, "%%--------------------------------------------------\n"
+	      "%%Stuff for day %2d being printed\n", date.tm_mday);
+      fprintf(out, "NextDay\n");
+
+      for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
+#ifdef USE_DB3
+	 if (use_db3_tags) {
+	    ret = db3_is_float(&(temp_al->ma.a), &category);
+	    /* jpilot_logf(LOG_DEBUG, "category = 0x%x\n", category); */
+	    cat_bit=1<<category;
+	    if (!(cat_bit & datebook_category)) {
+	       jpilot_logf(LOG_DEBUG, "skipping rec not in this "
+			   "category\n");
+	       continue;
+	    }
+	 }
+#endif
+	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
+	    char tmp[20];
+	    char datef1[20];
+	    char datef2[20];
+	    tmp[0]='\0';
+	    if ( ! temp_al->ma.a.event) {
+	       get_pref_time_no_secs(datef1);
+	       g_snprintf(datef2, 19, "(%s )", datef1);
+	       datef2[19]='\0';
+	       strftime(tmp, 19, datef2, &(temp_al->ma.a.begin));
+	       tmp[19]='\0';
+	    }
+	    desc[0]='\0';
+	    if (temp_al->ma.a.description) {
+	       ps_strncat(desc, temp_al->ma.a.description, 40);
+	       desc[40]='\0';
+	    }
+	    remove_cr_lfs(desc);
+	    fprintf(out, "%s (%s) %simedItem\n", tmp, desc,
+		    (strlen(tmp) == 0) ? "Unt" : "T" );
+	 }
+      }
+   }
+
+   /*------------------------------------------------------------------*/
+   memcpy(&date, date_in, sizeof(struct tm));
+   sub_months_from_date(&date, 1);
+   date.tm_mday = 1;	/* Go to the first of the month */
+   mktime(&date);
+   strftime(desc, 50, "(%B %Y) %w ", &date);
+   fprintf(out, "\n\n%%----------------------------------------\n"
+           "%% Now generate the small months\n\n"
+           "%s %d ", desc, days_in_mon(&date));
+
+   memcpy(&date, date_in, sizeof(struct tm));
+   add_months_to_date(&date, 1);
+   date.tm_mday = 1;	/* Go to the first of the month */
+   mktime(&date);
+   strftime(desc, 50, "(%B %Y) %w ", &date);
+   fprintf(out, "%s %d SmallMonths\n", desc, days_in_mon(&date));
+
+   /*------------------------------------------------------------------*/
+
+   free_AppointmentList(&a_list);
+
+   fprintf(out, "grestore\n");
+   print_logo(out, 20, 30, 0.35);
+   fprintf(out, "\nshowpage\n");
+
+   print_close(out);
+
+   return(0);
+}
+
+/*----------------------------------------------------------------------
+ * reset_first_last	Routine to reset max/min appointment times
+ *----------------------------------------------------------------------*/
+
+void reset_first_last(void)
+{
+   first_hour = 25;
+   first_min  = 61;
+   last_hour  = -1;
+   last_min   = -1;
+}
+
+/*----------------------------------------------------------------------
+ * check_first_last	Routine to track max/min appointment times
+ *----------------------------------------------------------------------*/
+
+void check_first_last(AppointmentList *al)
+{
+   struct tm *ApptTime;
+   ApptTime = &(al->ma.a.begin);
+   if (ApptTime->tm_hour == first_hour) {
+      if (ApptTime->tm_min < first_min) first_min = ApptTime->tm_min;
+   }
+   else if (ApptTime->tm_hour < first_hour) {
+      first_hour = ApptTime->tm_hour;
+      first_min  = ApptTime->tm_min;
+   }
+
+   ApptTime = &(al->ma.a.end);
+   if (ApptTime->tm_hour == last_hour) {
+      if (ApptTime->tm_min > last_min) last_min = ApptTime->tm_min;
+   } else if (ApptTime->tm_hour > last_hour) {
+      last_hour = ApptTime->tm_hour;
+      last_min  = ApptTime->tm_min;
+   }
+}
+
+/*----------------------------------------------------------------------
+ * print_weeks_appts	Function to print a weeks appointments onto a
+ *			weekly plan. We assume that date_in is the chosen
+ *			first day of the week.
+ *----------------------------------------------------------------------*/
+
+int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
+{
+   AppointmentList *a_list, *temp_al;
+   struct tm date;
+   struct tm *today_date;
+   char desc[256], short_date[32];
+   int n;
+   time_t ltime;
+#ifdef USE_DB3
+   int ret, category, cat_bit;
+   long use_db3_tags;
+
+   get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+#endif
+
+   /*------------------------------------------------------------------
+    * Set up the PostScript output file, and print the header to it.
+    *------------------------------------------------------------------*/
+   time(&ltime);
+   today_date = localtime(&ltime);
+
+   if (! (out = print_open())) return(-1);
+   /*------------------------------------------------------------------
+    * These are preferences for page size (passed in), first and last
+    * hours on the plan (default; scales if earlier or later are present),
+    * and whether to print dashes across the page.
+    *------------------------------------------------------------------*/
+   fprintf(out, "%%!PS-Adobe-2.0\n\n"
+	   "/PageSize (%s) def\n\n", PaperSizes[paper_size]);
+   fprintf(out, "/FirstHour  9 def\n"
+	   "/LastHour  22 def\n");
+   fprintf(out, "/Dashes true def\n");
+
+   print_week_header(out);
+
+   /*------------------------------------------------------------------
+    * Run through the appointments, looking for earliest and latest
+    *------------------------------------------------------------------*/
+   a_list = NULL;
+   get_days_appointments2(&a_list, NULL, 2, 2, 2);
+   reset_first_last();
+
+   memcpy(&date, date_in, sizeof(struct tm));
+   for (n = 0; n < 7; n++, add_days_to_date(&date, 1)) {
+      for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
+#ifdef USE_DB3
+	 if (use_db3_tags) {
+	    ret = db3_is_float(&(temp_al->ma.a), &category);
+	    cat_bit=1<<category;
+	    if (!(cat_bit & datebook_category)) continue;
+	 }
+#endif
+	 if (isApptOnDate(&(temp_al->ma.a), &date))
+	   if ( ! temp_al->ma.a.event)
+	     check_first_last(temp_al);
+      }
+   }
+   if (last_min > 0) last_hour++;
+
+   /*------------------------------------------------------------------
+    * Now put in the finishing touches to the header, and kick-start
+    * the printing process
+    *------------------------------------------------------------------*/
+
+   fprintf(out,
+	   "%%------------------------------------------------------------\n"
+	   "%% This is today's date, the date of printing, plus the hour\n"
+	   "%% before & after the first and last appointments, respectively\n"
+	   "%d %d %d %d %d startprinting\n\n",
+	   today_date->tm_mday, today_date->tm_mon + 1,
+	   today_date->tm_year + 1900, first_hour, last_hour);
+   fprintf(out, "( by %s version %s) show\n", PN, VERSION);
+
+   print_logo(out, 20, 30, 0.35);
+
+   /*------------------------------------------------------------------
+    * Run through the appointments, printing them out
+    *------------------------------------------------------------------*/
+   a_list = NULL;
+
+   /* Get all of the appointments */
+   get_days_appointments2(&a_list, NULL, 2, 2, 2);
+
+   /* iterate through seven days */
+   memcpy(&date, date_in, sizeof(struct tm));
+
+   for (n = 0; n < 7; n++, add_days_to_date(&date, 1)) {
+      strftime(short_date, 30, "%a, %d %b, %Y", &date);
+      fprintf(out, "%d startday\n(%s) dateline\n", n, short_date);
+
+      for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
+#ifdef USE_DB3
+	 if (use_db3_tags) {
+	    ret = db3_is_float(&(temp_al->ma.a), &category);
+	    jpilot_logf(LOG_DEBUG, "category = 0x%x\n", category);
+	    cat_bit=1<<category;
+	    if (!(cat_bit & datebook_category)) {
+	       jpilot_logf(LOG_DEBUG, "skip rec not in this category\n");
+	       continue;
+	    }
+	 }
+#endif
+	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
+	    bzero(desc, 256);
+	    bzero(short_date, 32);
+
+	    if ( ! temp_al->ma.a.event)
+	      {
+		 char t1[21], t2[21];
+		 int j;
+		 strftime(t1, 10, "%I:%M%p", &(temp_al->ma.a.begin));
+		 strftime(t2, 10, "%I:%M%p", &(temp_al->ma.a.end));
+		 sprintf(short_date, "(%s to %s) ", t1, t2);
+		 for (j=0; j<30;j++) short_date[j] =tolower(short_date[j]);
+	      }
+	    if (temp_al->ma.a.description) {
+	       ps_strncat(desc, temp_al->ma.a.description, 250);
+	       remove_cr_lfs(desc);
+	    }
+	    fprintf(out, "%s (%s) itemline\n", short_date, desc);
+	 }
+      }
+   }
+   free_AppointmentList(&a_list);
+   fprintf(out, "\nfinishprinting\n");
+   print_close(out);
+
+   return(0);
+}
+
+/*----------------------------------------------------------------------
  * 
  * Address code
  * 
- */
+ *----------------------------------------------------------------------*/
+
 int print_address_header()
 {
    time_t ltime;
@@ -612,13 +740,13 @@ int print_address_header()
    long ivalue;
    const char *svalue;
    char str[256];
-   
+
    time(&ltime);
    date = localtime(&ltime);
-   
+
    get_pref(PREF_SHORTDATE, &ivalue, &svalue);
    strftime(str, 250, svalue, date);
-   
+
    fprintf(out,
 	   "%%!PS-Adobe-2.0\n"
 	   "%%%%Creator: J-Pilot\n"
@@ -821,7 +949,7 @@ int print_address_header()
 	   );
    return 0;
 }
-   
+
 int print_addresses(AddressList *address_list)
 {
    long one_rec_per_page;
@@ -836,7 +964,7 @@ int print_addresses(AddressList *address_list)
    int order[22]={0,1,13,2,3,4,5,6,7,8,9,10,11,12,14,15,16,17,18,19,20,21
    };
    char str[100];
-      
+
    out = print_open();
    if (!out) {
       return -1;
@@ -849,7 +977,7 @@ int print_addresses(AddressList *address_list)
    get_address_app_info(&address_app_info);
 
    print_address_header();
-   
+
    if (address_app_info.sortByCompany) {
       show1=2; /*company */
       show2=0; /*last name */
@@ -867,7 +995,7 @@ int print_addresses(AddressList *address_list)
    for (temp_al = address_list; temp_al; temp_al=temp_al->next) {
 
       fprintf(out, "%cHLINE\n", FLAG_CHAR);
-      
+
       str[0]='\0';
       if (temp_al->ma.a.entry[show1] || temp_al->ma.a.entry[show2]) {
 	 if (temp_al->ma.a.entry[show1] && temp_al->ma.a.entry[show2]) {
@@ -884,7 +1012,7 @@ int print_addresses(AddressList *address_list)
       } else {
 	    strcpy(str, "-Unnamed-");
       }
-      
+
       courier_bold_12();
       fprintf(out, "%s\n", str);
       courier_12();
@@ -917,7 +1045,7 @@ int print_addresses(AddressList *address_list)
    }
 
    print_close(out);
-   
+
 #ifdef HAVE_LOCALE_H
    setlocale(LC_ALL,"");
 #endif
@@ -930,14 +1058,14 @@ int print_addresses(AddressList *address_list)
  * ToDo code
  * 
  */
-int print_todos(ToDoList *todo_list)
+int print_todos(ToDoList *todo_list, char *category_name)
 {
    long one_rec_per_page;
    long lines_between_recs;
    long ivalue;
    ToDoList *temp_l;
    struct ToDo *todo;
-   int i, indent;
+   int indent;
    const char *datef;
    char str[100];
    time_t ltime;
@@ -952,8 +1080,12 @@ int print_todos(ToDoList *todo_list)
    setlocale(LC_ALL,"C");
 #endif
 
-   print_address_header();
-   
+   fprintf(out, "%%!PS-Adobe-2.0\n\n"
+           "/PageSize (%s) def\n\n", PaperSizes[PAPER_Letter]);
+   print_common_header(out);
+   fprintf(out, "/CategoryName (%s) def\n", category_name);
+   print_todo_header(out);
+
    get_pref(PREF_PRINT_ONE_PER_PAGE, &one_rec_per_page, NULL);
    get_pref(PREF_NUM_BLANK_LINES, &lines_between_recs, NULL);
 
@@ -962,18 +1094,12 @@ int print_todos(ToDoList *todo_list)
    strftime(str, 80, datef, now);
    indent=strlen(str) + 8;
 
-   courier_12();
-
    for (temp_l = todo_list; temp_l; temp_l=temp_l->next) {
       todo = &(temp_l->mtodo.todo);
 
-      if (todo->complete) {
-	 fprintf(out, "%cCHECKEDBOX\n", FLAG_CHAR);
-      } else {
-	 fprintf(out, "%cCHECKBOX\n", FLAG_CHAR);
-      }
-      
-      fprintf(out, "   %d ", todo->priority);
+      fprintf(out, todo->complete ? "true  " : "false ");
+
+      fprintf(out, "%d ", todo->priority);
 
       if (todo->indefinite) {
 	 sprintf(str, "%s           ", "No Due");
@@ -981,37 +1107,29 @@ int print_todos(ToDoList *todo_list)
       } else {
 	 strftime(str, 20, datef, &(todo->due));
       }
-      fprintf(out, " %s  ", str);
+      fprintf(out, "(%s) ", str);
 
       if (todo->description) {
-	 f_indent_print(out, indent, todo->description);
+	 fprintf(out, "(%s) ", todo->description);
+      } else {
+         fprintf(out, "() ");
       }
 
       if ((todo->note) && todo->note[0]) {
-	 fprintf(out, "\n");
-	 for (i=indent; i>0; i--) {
-	    fprintf(out, " ");
-	 }
-	 fprintf(out, "Note:\n");
-	 for (i=indent; i>0; i--) {
-	    fprintf(out, " ");
-	 }
-	 f_indent_print(out, indent, todo->note);
-      }
-
-      fprintf(out, "\n");
-
-      if (one_rec_per_page) {
-	 fprintf(out, "%cLINEFEED\n", FLAG_CHAR);
+	 fprintf(out, "(%s)", todo->note);
       } else {
-	 for (i=lines_between_recs; i>0; i--) {
-	    fprintf(out, "\n");
-	 }
+         fprintf(out, "()");
       }
+      fprintf(out, " Todo\n");
+
+      if (one_rec_per_page) {	
+	 fprintf(out, "NewPage\n");	
+      }	
    }
+   fprintf(out, "showpage\n");
 
    print_close(out);
-   
+
 #ifdef HAVE_LOCALE_H
    setlocale(LC_ALL,"");
 #endif
@@ -1041,7 +1159,7 @@ int print_memos(MemoList *memo_list)
 #endif
 
    print_address_header();
-   
+
    get_pref(PREF_PRINT_ONE_PER_PAGE, &one_rec_per_page, NULL);
    get_pref(PREF_NUM_BLANK_LINES, &lines_between_recs, NULL);
 
@@ -1055,7 +1173,7 @@ int print_memos(MemoList *memo_list)
 	 f_indent_print(out, 0, memo->text);
 	 fprintf(out, "\n");
       }
-      
+
       if (one_rec_per_page) {
 	 fprintf(out, "%cLINEFEED\n", FLAG_CHAR);
       } else {
@@ -1070,6 +1188,6 @@ int print_memos(MemoList *memo_list)
 #endif
 
    print_close(out);
-   
+
    return 0;
 }
