@@ -1,6 +1,7 @@
 /* weekview_gui.c
+ * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2001 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +22,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include "print.h"
 #include "utils.h"
 #include "prefs.h"
 #include "log.h"
@@ -54,10 +56,31 @@ static void
    gtk_widget_destroy(data);
 }
 
+/*----------------------------------------------------------------------
+ * cb_week_print     This is where week printing is kicked off from
+ *----------------------------------------------------------------------*/
+
+static void cb_week_print(GtkWidget *widget, gpointer data)
+{
+   long paper_size;
+
+   jpilot_logf(LOG_DEBUG, "cb_week_print called\n");
+   if (print_gui(window, DATEBOOK, 2, 0x02) == DIALOG_SAID_PRINT) {
+      get_pref(PREF_PAPER_SIZE, &paper_size, NULL);
+      if (paper_size==1) {
+	 print_weeks_appts(&glob_week_date, PAPER_A4);
+      } else {
+	 print_weeks_appts(&glob_week_date, PAPER_Letter);
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+
 void freeze_weeks_appts()
 {
    int i;
-   
+
    for (i=0; i<8; i++) {
       gtk_text_freeze(GTK_TEXT(glob_week_texts[i]));
    }
@@ -66,7 +89,7 @@ void freeze_weeks_appts()
 void thaw_weeks_appts()
 {
    int i;
-   
+
    for (i=0; i<8; i++) {
       gtk_text_thaw(GTK_TEXT(glob_week_texts[i]));
    }
@@ -91,7 +114,7 @@ cb_week_move(GtkWidget *widget,
 int clear_weeks_appts(GtkWidget **day_texts)
 {
    int i;
-   
+
    for (i=0; i<8; i++) {
       gtk_text_set_point(GTK_TEXT(day_texts[i]), 0);
       gtk_text_forward_delete(GTK_TEXT(day_texts[i]),
@@ -166,7 +189,7 @@ int display_weeks_appts(struct tm *date_in, GtkWidget **day_texts)
    if (svalue==NULL) {
       svalue = default_date;
    }
-   
+
    for (i=0; i<8; i++, add_days_to_date(&date, 1)) {
       strftime(short_date, 30, svalue, &date);
       g_snprintf(str, 80, "%s %s\n", _(days[(i + fdow)%7]), short_date);
@@ -217,7 +240,7 @@ int display_weeks_appts(struct tm *date_in, GtkWidget **day_texts)
       }
    }
    free_AppointmentList(&a_list);
-   
+
    return 0;
 }
 
@@ -233,7 +256,7 @@ void weekview_gui(struct tm *date_in)
    long fdow;
    int i;
    char title[200];
-   
+
    if (window) {
       return;
    }
@@ -276,6 +299,12 @@ void weekview_gui(struct tm *date_in)
 		      GTK_SIGNAL_FUNC(cb_quit), window);
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
 
+   /* Create a "Print" button */
+   button = gtk_button_new_with_label(_("Print"));
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_week_print), window);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
+
    /*Make a right arrow for going forward a week */
    button = gtk_button_new();
    arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
